@@ -24,6 +24,8 @@ function octaviaclient_install {
     if use_library_from_git "python-octaviaclient"; then
         git_clone_by_name "python-octaviaclient"
         setup_dev_lib "python-octaviaclient"
+    else
+        pip_install_gr python-octaviaclient
     fi
 }
 
@@ -162,6 +164,10 @@ function _stop_octavia_apache_wsgi {
 function create_octavia_accounts {
     create_service_user $OCTAVIA
 
+    # Increase the service account secgroups quota
+    # This is imporant for concurrent tempest testing
+    openstack quota set --secgroups 100 $SERVICE_PROJECT_NAME
+
     local octavia_service=$(get_or_create_service "octavia" \
         $OCTAVIA_SERVICE_TYPE "Octavia Load Balancing Service")
 
@@ -215,6 +221,7 @@ function octavia_configure {
     iniset $OCTAVIA_CONF controller_worker amphora_driver ${OCTAVIA_AMPHORA_DRIVER}
     iniset $OCTAVIA_CONF controller_worker compute_driver ${OCTAVIA_COMPUTE_DRIVER}
     iniset $OCTAVIA_CONF controller_worker network_driver ${OCTAVIA_NETWORK_DRIVER}
+    iniset $OCTAVIA_CONF controller_worker amp_image_tag ${OCTAVIA_AMP_IMAGE_TAG}
 
     iniuncomment $OCTAVIA_CONF health_manager heartbeat_key
     iniset $OCTAVIA_CONF health_manager heartbeat_key ${OCTAVIA_HEALTH_KEY}
@@ -248,6 +255,9 @@ function octavia_configure {
     iniuncomment $OCTAVIA_CONF controller_worker amp_active_retries
     iniuncomment $OCTAVIA_CONF controller_worker amp_active_wait_sec
     iniuncomment $OCTAVIA_CONF controller_worker workers
+    iniuncomment $OCTAVIA_CONF controller_worker loadbalancer_topology
+
+    iniset $OCTAVIA_CONF controller_worker loadbalancer_topology ${OCTAVIA_LB_TOPOLOGY}
 
     # devstack optimizations for tempest runs
     iniset $OCTAVIA_CONF haproxy_amphora connection_max_retries 1500
@@ -295,6 +305,10 @@ function octavia_configure {
     iniset $OCTAVIA_CONF certificates ca_private_key_passphrase foobar
     iniset $OCTAVIA_CONF certificates server_certs_key_passphrase insecure-key-do-not-use-this-key
 
+    if [[ "$OCTAVIA_USE_LEGACY_RBAC" == "True" ]]; then
+        cp $OCTAVIA_DIR/etc/policy/admin_or_owner-policy.json $OCTAVIA_CONF_DIR/policy.json
+    fi
+
     # create dhclient.conf file for dhclient
     sudo mkdir -m755 -p $OCTAVIA_DHCLIENT_DIR
     sudo cp $OCTAVIA_DIR/etc/dhcp/dhclient.conf $OCTAVIA_DHCLIENT_CONF
@@ -307,6 +321,17 @@ function octavia_configure {
         fi
     fi
 
+    if [ $OCTAVIA_NODE == 'main' ]; then
+        configure_octavia_api_haproxy
+        # make sure octavia is reachable from haproxy
+        iniset $OCTAVIA_CONF DEFAULT bind_port ${OCTAVIA_HA_PORT}
+        iniset $OCTAVIA_CONF DEFAULT bind_host 0.0.0.0
+    fi
+    if [ $OCTAVIA_NODE != 'main' ] && [ $OCTAVIA_NODE != 'standalone' ] ; then
+        # make sure octavia is reachable from haproxy from main node
+        iniset $OCTAVIA_CONF DEFAULT bind_port ${OCTAVIA_HA_PORT}
+        iniset $OCTAVIA_CONF DEFAULT bind_host 0.0.0.0
+    fi
 }
 
 function create_mgmt_network_interface {
@@ -322,9 +347,9 @@ function create_mgmt_network_interface {
     MGMT_PORT_IP=$(openstack port show -f value -c fixed_ips $MGMT_PORT_ID | awk '{FS=",| "; gsub(",",""); gsub("'\''",""); for(i = 1; i <= NF; ++i) {if ($i ~ /^ip_address/) {n=index($i, "="); if (substr($i, n+1) ~ "\\.") print substr($i, n+1)}}}')
     if function_exists octavia_create_network_interface_device ; then
         octavia_create_network_interface_device o-hm0 $MGMT_PORT_ID $MGMT_PORT_MAC
-    elif [[ $Q_AGENT == "openvswitch" ]]; then
+    elif [[ $NEUTRON_AGENT == "openvswitch" || $Q_AGENT == "openvswitch" ]]; then
         sudo ovs-vsctl -- --may-exist add-port ${OVS_BRIDGE:-br-int} o-hm0 -- set Interface o-hm0 type=internal -- set Interface o-hm0 external-ids:iface-status=active -- set Interface o-hm0 external-ids:attached-mac=$MGMT_PORT_MAC -- set Interface o-hm0 external-ids:iface-id=$MGMT_PORT_ID -- set Interface o-hm0 external-ids:skip_cleanup=true
-    elif [[ $Q_AGENT == "linuxbridge" ]]; then
+    elif [[ $NEUTRON_AGENT == "linuxbridge" || $Q_AGENT == "linuxbridge" ]]; then
         if ! ip link show o-hm0 ; then
             sudo ip link add o-hm0 type veth peer name o-bhm0
             NETID=$(openstack network show lb-mgmt-net -c id -f value)
@@ -336,7 +361,6 @@ function create_mgmt_network_interface {
         die "Unknown network controller. Please define octavia_create_network_interface_device"
     fi
     sudo ip link set dev o-hm0 address $MGMT_PORT_MAC
-    sudo dhclient -v o-hm0 -cf $OCTAVIA_DHCLIENT_CONF
     sudo iptables -I INPUT -i o-hm0 -p udp --dport 5555 -j ACCEPT
 
     if [ $OCTAVIA_CONTROLLER_IP_PORT_LIST == 'auto' ] ; then
@@ -360,14 +384,18 @@ function build_mgmt_network {
     openstack security group rule create --protocol icmp lb-mgmt-sec-grp
     openstack security group rule create --protocol tcp --dst-port 22 lb-mgmt-sec-grp
     openstack security group rule create --protocol tcp --dst-port 9443 lb-mgmt-sec-grp
-    openstack security group rule create --protocol icmpv6 --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
-    openstack security group rule create --protocol tcp --dst-port 22 --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
-    openstack security group rule create --protocol tcp --dst-port 9443 --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
+    if [ $IPV6_ENABLED == 'true' ] ; then
+        openstack security group rule create --protocol icmpv6 --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
+        openstack security group rule create --protocol tcp --dst-port 22 --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
+        openstack security group rule create --protocol tcp --dst-port 9443 --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
+    fi
 
     # Create security group and rules
     openstack security group create lb-health-mgr-sec-grp
     openstack security group rule create --protocol udp --dst-port $OCTAVIA_HM_LISTEN_PORT lb-health-mgr-sec-grp
-    openstack security group rule create --protocol udp --dst-port $OCTAVIA_HM_LISTEN_PORT --ethertype IPv6 --remote-ip ::/0 lb-health-mgr-sec-grp
+    if [ $IPV6_ENABLED == 'true' ] ; then
+        openstack security group rule create --protocol udp --dst-port $OCTAVIA_HM_LISTEN_PORT --ethertype IPv6 --remote-ip ::/0 lb-health-mgr-sec-grp
+    fi
 }
 
 function configure_lb_mgmt_sec_grp {
@@ -413,79 +441,12 @@ function configure_octavia_api_haproxy {
 
 function octavia_start {
 
-    # Several steps in this function would more logically be in the configure function, but
-    # we need nova, glance, and neutron to be running.
-
-    if [ $OCTAVIA_NODE != 'main' ] && [ $OCTAVIA_NODE != 'standalone' ]  && [ $OCTAVIA_NODE != 'api' ]; then
-        # without the other services enabled apparently we don't have
-        # credentials at this point
-        TOP_DIR=$(cd $(dirname "$0") && pwd)
-        source ${TOP_DIR}/openrc admin admin
+    if  ! ps aux | grep -q [o]-hm0 && [ $OCTAVIA_NODE != 'api' ] ; then
+        sudo dhclient -v o-hm0 -cf $OCTAVIA_DHCLIENT_CONF
     fi
 
-    if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
-        # things that should only happen on the ha main node / or once
-        openstack keypair create --public-key ${OCTAVIA_AMP_SSH_KEY_PATH}.pub ${OCTAVIA_AMP_SSH_KEY_NAME}
-
-        # Check if an amphora image is already loaded
-        AMPHORA_IMAGE_NAME=$(openstack image list --property name=${OCTAVIA_AMP_IMAGE_NAME} -f value -c Name)
-        export AMPHORA_IMAGE_NAME
-
-        if [ "$AMPHORA_IMAGE_NAME" == ${OCTAVIA_AMP_IMAGE_NAME} ]; then
-            echo "Found existing amphora image: $AMPHORA_IMAGE_NAME"
-            echo "Skipping amphora image build"
-            export DISABLE_AMP_IMAGE_BUILD=True
-        fi
-
-        if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
-            build_octavia_worker_image
-        fi
-
-        OCTAVIA_AMP_IMAGE_ID=$(openstack image list -f value --property name=${OCTAVIA_AMP_IMAGE_NAME} -c ID)
-
-        if [ -n "$OCTAVIA_AMP_IMAGE_ID" ]; then
-            openstack image set --tag ${OCTAVIA_AMP_IMAGE_TAG} ${OCTAVIA_AMP_IMAGE_ID}
-        fi
-
-        # Create a management network.
-        build_mgmt_network
-
-        create_octavia_accounts
-
-        # Adds service and endpoint
-        if is_service_enabled tempest; then
-            configure_octavia_tempest ${OCTAVIA_AMP_NETWORK_ID}
-        fi
-    fi
-
-    if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
-        set_octavia_worker_image_owner_id
-    fi
-
-    if [ $OCTAVIA_NODE != 'api' ] ; then
-        create_mgmt_network_interface
-
-        create_amphora_flavor
-        configure_lb_mgmt_sec_grp
-    fi
-
-    iniset $OCTAVIA_CONF controller_worker amp_image_tag ${OCTAVIA_AMP_IMAGE_TAG}
-
-    OCTAVIA_AMP_NETWORK_ID=$(openstack network show lb-mgmt-net -f value -c id)
-
-    iniset $OCTAVIA_CONF controller_worker amp_boot_network_list ${OCTAVIA_AMP_NETWORK_ID}
-
     if [ $OCTAVIA_NODE == 'main' ]; then
-        configure_octavia_api_haproxy
         run_process $OCTAVIA_API_HAPROXY "/usr/sbin/haproxy -db -V -f ${OCTAVIA_CONF_DIR}/haproxy.cfg"
-        # make sure octavia is reachable from haproxy
-        iniset $OCTAVIA_CONF DEFAULT bind_port ${OCTAVIA_HA_PORT}
-        iniset $OCTAVIA_CONF DEFAULT bind_host 0.0.0.0
-    fi
-    if [ $OCTAVIA_NODE != 'main' ] && [ $OCTAVIA_NODE != 'standalone' ] ; then
-        # make sure octavia is reachable from haproxy from main node
-        iniset $OCTAVIA_CONF DEFAULT bind_port ${OCTAVIA_HA_PORT}
-        iniset $OCTAVIA_CONF DEFAULT bind_host 0.0.0.0
     fi
 
     if [[ "$OCTAVIA_USE_MOD_WSGI" == "True" ]]; then
@@ -497,11 +458,6 @@ function octavia_start {
     run_process $OCTAVIA_CONSUMER  "$OCTAVIA_CONSUMER_BINARY $OCTAVIA_CONSUMER_ARGS"
     run_process $OCTAVIA_HOUSEKEEPER  "$OCTAVIA_HOUSEKEEPER_BINARY $OCTAVIA_HOUSEKEEPER_ARGS"
     run_process $OCTAVIA_HEALTHMANAGER  "$OCTAVIA_HEALTHMANAGER_BINARY $OCTAVIA_HEALTHMANAGER_ARGS"
-
-    if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
-        add_load-balancer_roles
-    fi
-
 }
 
 function octavia_stop {
@@ -516,13 +472,13 @@ function octavia_stop {
     stop_process $OCTAVIA_HEALTHMANAGER
 
     # Kill dhclient process started for o-hm0 interface
-    pids=$(ps aux | awk '/o-hm0/ { print $2 }')
+    pids=$(ps aux | awk '/[o]-hm0/ { print $2 }')
     [ ! -z "$pids" ] && sudo kill $pids
     if function_exists octavia_delete_network_interface_device ; then
         octavia_delete_network_interface_device o-hm0
-    elif [[ $Q_AGENT == "openvswitch" ]]; then
+    elif [[ $NEUTRON_AGENT == "openvswitch" || $Q_AGENT == "openvswitch" ]]; then
         :  # Do nothing
-    elif [[ $Q_AGENT == "linuxbridge" ]]; then
+    elif [[ $NEUTRON_AGENT == "linuxbridge" || $Q_AGENT == "linuxbridge" ]]; then
         if ip link show o-hm0 ; then
             sudo ip link del o-hm0
         fi
@@ -569,11 +525,80 @@ function add_load-balancer_roles {
     openstack role add --user demo --project demo load-balancer_member
 }
 
+function octavia_init {
+   if [ $OCTAVIA_NODE != 'main' ] && [ $OCTAVIA_NODE != 'standalone' ]  && [ $OCTAVIA_NODE != 'api' ]; then
+       # without the other services enabled apparently we don't have
+       # credentials at this point
+       TOP_DIR=$(cd $(dirname "$0") && pwd)
+       source ${TOP_DIR}/openrc admin admin
+   fi
+
+   if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
+       # things that should only happen on the ha main node / or once
+       if ! openstack keypair show ${OCTAVIA_AMP_SSH_KEY_NAME} ; then
+           openstack keypair create --public-key ${OCTAVIA_AMP_SSH_KEY_PATH}.pub ${OCTAVIA_AMP_SSH_KEY_NAME}
+       fi
+
+       # Check if an amphora image is already loaded
+       AMPHORA_IMAGE_NAME=$(openstack image list --property name=${OCTAVIA_AMP_IMAGE_NAME} -f value -c Name)
+       export AMPHORA_IMAGE_NAME
+
+       if [ "$AMPHORA_IMAGE_NAME" == ${OCTAVIA_AMP_IMAGE_NAME} ]; then
+           echo "Found existing amphora image: $AMPHORA_IMAGE_NAME"
+           echo "Skipping amphora image build"
+           export DISABLE_AMP_IMAGE_BUILD=True
+       fi
+
+       if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
+           build_octavia_worker_image
+       fi
+
+       OCTAVIA_AMP_IMAGE_ID=$(openstack image list -f value --property name=${OCTAVIA_AMP_IMAGE_NAME} -c ID)
+
+       if [ -n "$OCTAVIA_AMP_IMAGE_ID" ]; then
+           openstack image set --tag ${OCTAVIA_AMP_IMAGE_TAG} ${OCTAVIA_AMP_IMAGE_ID}
+       fi
+
+       # Create a management network.
+       build_mgmt_network
+       OCTAVIA_AMP_NETWORK_ID=$(openstack network show lb-mgmt-net -f value -c id)
+       iniset $OCTAVIA_CONF controller_worker amp_boot_network_list ${OCTAVIA_AMP_NETWORK_ID}
+
+       create_octavia_accounts
+
+       # Adds service and endpoint
+       if is_service_enabled tempest; then
+           configure_octavia_tempest ${OCTAVIA_AMP_NETWORK_ID}
+       fi
+
+       add_load-balancer_roles
+   elif [ $OCTAVIA_NODE == 'api' ] ; then
+       create_octavia_accounts
+
+       # Adds service and endpoint
+       if is_service_enabled tempest; then
+           configure_octavia_tempest ${OCTAVIA_AMP_NETWORK_ID}
+       fi
+
+       add_load-balancer_roles
+   fi
+
+   if [ $OCTAVIA_NODE != 'api' ] ; then
+       create_mgmt_network_interface
+       create_amphora_flavor
+       configure_lb_mgmt_sec_grp
+   fi
+
+   if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
+       set_octavia_worker_image_owner_id
+   fi
+}
+
 # check for service enabled
 if is_service_enabled $OCTAVIA; then
     if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then # main-ha node stuff only
-        if ! is_service_enabled $Q_SVC; then
-            die "The neutron $Q_SVC service must be enabled to use $OCTAVIA"
+        if ! is_service_enabled $NEUTRON_ANY; then
+            die "The neutron-api/q-svc service must be enabled to use $OCTAVIA"
         fi
 
         if [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
@@ -597,7 +622,10 @@ if is_service_enabled $OCTAVIA; then
 
     elif [[ "$1" == "stack" && "$2" == "extra" ]]; then
         # Initialize and start the octavia service
-        echo_summary "Initializing octavia"
+        echo_summary "Initializing Octavia"
+        octavia_init
+
+        echo_summary "Starting Octavia"
         octavia_start
     fi
 fi
