@@ -12,15 +12,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import logging
-
 from oslo_config import cfg
 from oslo_db import exception as odb_exceptions
+from oslo_log import log as logging
 from oslo_utils import excutils
 import pecan
 from wsme import types as wtypes
 from wsmeext import pecan as wsme_pecan
 
+from octavia.api.drivers import data_models as driver_dm
+from octavia.api.drivers import driver_factory
+from octavia.api.drivers import utils as driver_utils
 from octavia.api.v2.controllers import base
 from octavia.api.v2.controllers import l7rule
 from octavia.api.v2.types import l7policy as l7policy_types
@@ -30,6 +32,7 @@ from octavia.common import exceptions
 from octavia.common import validate
 from octavia.db import api as db_api
 from octavia.db import prepare as db_prepare
+from octavia.i18n import _
 
 
 CONF = cfg.CONF
@@ -41,14 +44,14 @@ class L7PolicyController(base.BaseController):
 
     def __init__(self):
         super(L7PolicyController, self).__init__()
-        self.handler = self.handler.l7policy
 
     @wsme_pecan.wsexpose(l7policy_types.L7PolicyRootResponse, wtypes.text,
                          [wtypes.text], ignore_extra_args=True)
     def get(self, id, fields=None):
         """Gets a single l7policy's details."""
         context = pecan.request.context.get('octavia_context')
-        db_l7policy = self._get_db_l7policy(context.session, id)
+        db_l7policy = self._get_db_l7policy(context.session, id,
+                                            show_deleted=False)
 
         self._auth_validate_action(context, db_l7policy.project_id,
                                    constants.RBAC_GET_ONE)
@@ -90,12 +93,6 @@ class L7PolicyController(base.BaseController):
             raise exceptions.ImmutableObject(resource='Load Balancer',
                                              id=lb_id)
 
-    def _get_listener_and_loadbalancer_id(self, db_l7policy):
-        """Get listener and loadbalancer ids from the l7policy db_model."""
-        load_balancer_id = db_l7policy.listener.load_balancer_id
-        listener_id = db_l7policy.listener_id
-        return load_balancer_id, listener_id
-
     def _reset_lb_and_listener_statuses(self, session, lb_id, listener_id):
         # Setting LB + listeners back to active because this should be a
         # recoverable error
@@ -118,25 +115,11 @@ class L7PolicyController(base.BaseController):
             # do not give any information as to what constraint failed
             raise exceptions.InvalidOption(value='', option='')
 
-    def _send_l7policy_to_handler(self, session, db_l7policy, lb_id):
-        try:
-            LOG.info("Sending Creation of L7Policy %s to handler",
-                     db_l7policy.id)
-            self.handler.create(db_l7policy)
-        except Exception:
-            with excutils.save_and_reraise_exception(
-                    reraise=False), db_api.get_lock_session() as lock_session:
-                self._reset_lb_and_listener_statuses(
-                    lock_session, lb_id=lb_id,
-                    listener_id=db_l7policy.listener_id)
-                # L7Policy now goes to ERROR
-                self.repositories.l7policy.update(
-                    lock_session, db_l7policy.id,
-                    provisioning_status=constants.ERROR)
-        db_l7policy = self._get_db_l7policy(session, db_l7policy.id)
-        result = self._convert_db_to_type(db_l7policy,
-                                          l7policy_types.L7PolicyResponse)
-        return l7policy_types.L7PolicyRootResponse(l7policy=result)
+    def _escape_l7policy_udp_pool_request(self, pool):
+        if pool.protocol == constants.PROTOCOL_UDP:
+            raise exceptions.ValidationException(
+                detail=_("%s protocol pool can not be assigned to "
+                         "l7policy.") % constants.PROTOCOL_UDP)
 
     @wsme_pecan.wsexpose(l7policy_types.L7PolicyRootResponse,
                          body=l7policy_types.L7PolicyRootPOST, status_code=201)
@@ -146,18 +129,23 @@ class L7PolicyController(base.BaseController):
         context = pecan.request.context.get('octavia_context')
         # Make sure any pool specified by redirect_pool_id exists
         if l7policy.redirect_pool_id:
-            self._get_db_pool(
+            db_pool = self._get_db_pool(
                 context.session, l7policy.redirect_pool_id)
+            self._escape_l7policy_udp_pool_request(db_pool)
         # Verify the parent listener exists
         listener_id = l7policy.listener_id
         listener = self._get_db_listener(
             context.session, listener_id)
         load_balancer_id = listener.load_balancer_id
-        l7policy.project_id = listener.project_id
+        l7policy.project_id, provider = self._get_lb_project_id_provider(
+            context.session, load_balancer_id)
 
         self._auth_validate_action(context, l7policy.project_id,
                                    constants.RBAC_POST)
 
+        # Load the driver early as it also provides validation
+        driver = driver_factory.get_driver(provider)
+
         lock_session = db_api.get_session(autocommit=False)
         try:
             if self.repositories.check_quota_met(
@@ -177,13 +165,26 @@ class L7PolicyController(base.BaseController):
                 listener_ids=[listener_id])
             db_l7policy = self._validate_create_l7policy(
                 lock_session, l7policy_dict)
+
+            # Prepare the data for the driver data model
+            provider_l7policy = (
+                driver_utils.db_l7policy_to_provider_l7policy(db_l7policy))
+
+            # Dispatch to the driver
+            LOG.info("Sending create L7 Policy %s to provider %s",
+                     db_l7policy.id, driver.name)
+            driver_utils.call_provider(
+                driver.name, driver.l7policy_create, provider_l7policy)
+
             lock_session.commit()
         except Exception:
             with excutils.save_and_reraise_exception():
                 lock_session.rollback()
 
-        return self._send_l7policy_to_handler(context.session, db_l7policy,
-                                              lb_id=load_balancer_id)
+        db_l7policy = self._get_db_l7policy(context.session, db_l7policy.id)
+        result = self._convert_db_to_type(db_l7policy,
+                                          l7policy_types.L7PolicyResponse)
+        return l7policy_types.L7PolicyRootResponse(l7policy=result)
 
     def _graph_create(self, lock_session, policy_dict):
         load_balancer_id = policy_dict.pop('load_balancer_id', None)
@@ -200,6 +201,7 @@ class L7PolicyController(base.BaseController):
                 l7rule.L7RuleController(db_policy.id)._graph_create(
                     lock_session, r))
 
+        db_policy.l7rules = new_rules
         return db_policy
 
     @wsme_pecan.wsexpose(l7policy_types.L7PolicyRootResponse,
@@ -218,36 +220,54 @@ class L7PolicyController(base.BaseController):
         context = pecan.request.context.get('octavia_context')
         # Make sure any specified redirect_pool_id exists
         if l7policy_dict.get('redirect_pool_id'):
-            self._get_db_pool(
+            db_pool = self._get_db_pool(
                 context.session, l7policy_dict['redirect_pool_id'])
-        db_l7policy = self._get_db_l7policy(context.session, id)
+            self._escape_l7policy_udp_pool_request(db_pool)
+        db_l7policy = self._get_db_l7policy(context.session, id,
+                                            show_deleted=False)
         load_balancer_id, listener_id = self._get_listener_and_loadbalancer_id(
             db_l7policy)
+        project_id, provider = self._get_lb_project_id_provider(
+            context.session, load_balancer_id)
 
-        self._auth_validate_action(context, db_l7policy.project_id,
-                                   constants.RBAC_PUT)
+        self._auth_validate_action(context, project_id, constants.RBAC_PUT)
 
-        self._test_lb_and_listener_statuses(context.session,
-                                            lb_id=load_balancer_id,
-                                            listener_ids=[listener_id])
-        self.repositories.l7policy.update(
-            context.session, db_l7policy.id,
-            provisioning_status=constants.PENDING_UPDATE)
+        # Load the driver early as it also provides validation
+        driver = driver_factory.get_driver(provider)
 
-        try:
-            LOG.info("Sending Update of L7Policy %s to handler", id)
-            self.handler.update(
-                db_l7policy, sanitized_l7policy)
-        except Exception:
-            with excutils.save_and_reraise_exception(
-                    reraise=False), db_api.get_lock_session() as lock_session:
-                self._reset_lb_and_listener_statuses(
-                    lock_session, lb_id=load_balancer_id,
-                    listener_id=db_l7policy.listener_id)
-                # L7Policy now goes to ERROR
-                self.repositories.l7policy.update(
-                    lock_session, db_l7policy.id,
-                    provisioning_status=constants.ERROR)
+        with db_api.get_lock_session() as lock_session:
+
+            self._test_lb_and_listener_statuses(lock_session,
+                                                lb_id=load_balancer_id,
+                                                listener_ids=[listener_id])
+
+            # Prepare the data for the driver data model
+            l7policy_dict = sanitized_l7policy.to_dict(render_unsets=False)
+            l7policy_dict['id'] = id
+            provider_l7policy_dict = (
+                driver_utils.l7policy_dict_to_provider_dict(l7policy_dict))
+
+            # Also prepare the baseline object data
+            old_provider_l7policy = (
+                driver_utils.db_l7policy_to_provider_l7policy(db_l7policy))
+
+            # Dispatch to the driver
+            LOG.info("Sending update L7 Policy %s to provider %s",
+                     id, driver.name)
+            driver_utils.call_provider(
+                driver.name, driver.l7policy_update,
+                old_provider_l7policy,
+                driver_dm.L7Policy.from_dict(provider_l7policy_dict))
+
+            # Update the database to reflect what the driver just accepted
+            sanitized_l7policy.provisioning_status = constants.PENDING_UPDATE
+            db_l7policy_dict = sanitized_l7policy.to_dict(render_unsets=False)
+            self.repositories.l7policy.update(lock_session, id,
+                                              **db_l7policy_dict)
+
+        # Force SQL alchemy to query the DB, otherwise we get inconsistent
+        # results
+        context.session.expire_all()
         db_l7policy = self._get_db_l7policy(context.session, id)
         result = self._convert_db_to_type(db_l7policy,
                                           l7policy_types.L7PolicyResponse)
@@ -257,37 +277,36 @@ class L7PolicyController(base.BaseController):
     def delete(self, id):
         """Deletes a l7policy."""
         context = pecan.request.context.get('octavia_context')
-        db_l7policy = self._get_db_l7policy(context.session, id)
+        db_l7policy = self._get_db_l7policy(context.session, id,
+                                            show_deleted=False)
         load_balancer_id, listener_id = self._get_listener_and_loadbalancer_id(
             db_l7policy)
+        project_id, provider = self._get_lb_project_id_provider(
+            context.session, load_balancer_id)
 
-        self._auth_validate_action(context, db_l7policy.project_id,
-                                   constants.RBAC_DELETE)
+        self._auth_validate_action(context, project_id, constants.RBAC_DELETE)
 
         if db_l7policy.provisioning_status == constants.DELETED:
             return
 
-        self._test_lb_and_listener_statuses(context.session,
-                                            lb_id=load_balancer_id,
-                                            listener_ids=[listener_id])
-        self.repositories.l7policy.update(
-            context.session, db_l7policy.id,
-            provisioning_status=constants.PENDING_DELETE)
+        # Load the driver early as it also provides validation
+        driver = driver_factory.get_driver(provider)
 
-        try:
-            LOG.info("Sending Deletion of L7Policy %s to handler",
-                     db_l7policy.id)
-            self.handler.delete(db_l7policy)
-        except Exception:
-            with excutils.save_and_reraise_exception(
-                    reraise=False), db_api.get_lock_session() as lock_session:
-                self._reset_lb_and_listener_statuses(
-                    lock_session, lb_id=load_balancer_id,
-                    listener_id=db_l7policy.listener_id)
-                # L7Policy now goes to ERROR
-                self.repositories.l7policy.update(
-                    lock_session, db_l7policy.id,
-                    provisioning_status=constants.ERROR)
+        with db_api.get_lock_session() as lock_session:
+
+            self._test_lb_and_listener_statuses(lock_session,
+                                                lb_id=load_balancer_id,
+                                                listener_ids=[listener_id])
+            self.repositories.l7policy.update(
+                lock_session, db_l7policy.id,
+                provisioning_status=constants.PENDING_DELETE)
+
+            LOG.info("Sending delete L7 Policy %s to provider %s",
+                     id, driver.name)
+            provider_l7policy = driver_utils.db_l7policy_to_provider_l7policy(
+                db_l7policy)
+            driver_utils.call_provider(driver.name, driver.l7policy_delete,
+                                       provider_l7policy)
 
     @pecan.expose()
     def _lookup(self, l7policy_id, *remainder):
@@ -297,7 +316,7 @@ class L7PolicyController(base.BaseController):
         which controller, if any, should control be passed.
         """
         context = pecan.request.context.get('octavia_context')
-        if l7policy_id and len(remainder) and remainder[0] == 'rules':
+        if l7policy_id and remainder and remainder[0] == 'rules':
             remainder = remainder[1:]
             db_l7policy = self.repositories.l7policy.get(
                 context.session, id=l7policy_id)
@@ -307,3 +326,4 @@ class L7PolicyController(base.BaseController):
                     resource='L7Policy', id=l7policy_id)
             return l7rule.L7RuleController(
                 l7policy_id=db_l7policy.id), remainder
+        return None
