@@ -82,6 +82,7 @@ MAX_PORT_NUMBER = 65535
 DEFAULT_CONNECTION_LIMIT = -1
 MIN_CONNECTION_LIMIT = -1
 
+DEFAULT_WEIGHT = 1
 MIN_WEIGHT = 0
 MAX_WEIGHT = 256
 
@@ -466,6 +467,7 @@ SYSCTL_CMD = '/sbin/sysctl'
 AMP_ACTION_START = 'start'
 AMP_ACTION_STOP = 'stop'
 AMP_ACTION_RELOAD = 'reload'
+AMP_ACTION_RESTART = 'restart'
 GLANCE_IMAGE_ACTIVE = 'active'
 
 INIT_SYSTEMD = 'systemd'
