@@ -12,6 +12,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import ipaddress
 import time
 
 from neutronclient.common import exceptions as neutron_client_exceptions
@@ -23,16 +24,16 @@ import six
 from octavia.common import clients
 from octavia.common import constants
 from octavia.common import data_models
+from octavia.common import exceptions
+from octavia.i18n import _
 from octavia.network import base
 from octavia.network import data_models as n_data_models
 from octavia.network.drivers.neutron import base as neutron_base
 from octavia.network.drivers.neutron import utils
 
-import ipaddress
-
-
 LOG = logging.getLogger(__name__)
 AAP_EXT_ALIAS = 'allowed-address-pairs'
+PROJECT_ID_ALIAS = 'project-id'
 VIP_SECURITY_GRP_PREFIX = 'lb-'
 OCTAVIA_OWNER = 'Octavia'
 
@@ -82,6 +83,7 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                     is_correct_interface = False
             if is_correct_interface:
                 return interface
+        return None
 
     def _plug_amphora_vip(self, amphora, subnet):
         # We need a vip port owned by Octavia for Act/Stby and failover
@@ -97,6 +99,13 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             LOG.debug('Created vip port: %(port_id)s for amphora: %(amp)s',
                       {'port_id': new_port.id, 'amp': amphora.id})
 
+        except Exception:
+            message = _('Error creating the base (VRRP) port for the VIP with '
+                        'port details: {}').format(port)
+            LOG.exception(message)
+            raise base.PlugVIPException(message)
+
+        try:
             interface = self.plug_port(amphora, new_port)
         except Exception:
             message = _('Error plugging amphora (compute_id: {compute_id}) '
@@ -104,6 +113,16 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                             compute_id=amphora.compute_id,
                             network_id=subnet.network_id)
             LOG.exception(message)
+            try:
+                if new_port:
+                    self.neutron_client.delete_port(new_port.id)
+                    LOG.debug('Deleted base (VRRP) port %s due to plug_port '
+                              'failure.', new_port.id)
+            except Exception:
+                LOG.exception('Failed to delete base (VRRP) port %s after '
+                              'plug_port failed. This resource is being '
+                              'abandoned and should be manually deleted when '
+                              'neutron is functional.', new_port.id)
             raise base.PlugVIPException(message)
         return interface
 
@@ -111,7 +130,7 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         try:
             self._add_allowed_address_pair_to_port(port_id, vip_address)
         except neutron_client_exceptions.PortNotFoundClient as e:
-            raise base.PortNotFound(e.message)
+            raise base.PortNotFound(str(e))
         except Exception:
             message = _('Error adding allowed address pair {ip} '
                         'to port {port_id}.').format(ip=vip_address,
@@ -124,39 +143,52 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         sec_grps = self.neutron_client.list_security_groups(name=sec_grp_name)
         if sec_grps and sec_grps.get('security_groups'):
             return sec_grps.get('security_groups')[0]
+        return None
 
     def _get_ethertype_for_ip(self, ip):
         address = ipaddress.ip_address(
             ip if isinstance(ip, six.text_type) else six.u(ip))
-        return 'IPv6' if address.version is 6 else 'IPv4'
+        return 'IPv6' if address.version == 6 else 'IPv4'
 
     def _update_security_group_rules(self, load_balancer, sec_grp_id):
         rules = self.neutron_client.list_security_group_rules(
             security_group_id=sec_grp_id)
         updated_ports = [
-            listener.protocol_port for listener in load_balancer.listeners
+            (listener.protocol_port,
+             constants.PROTOCOL_TCP.lower()
+             if listener.protocol != constants.PROTOCOL_UDP else
+             constants.PROTOCOL_UDP.lower())
+            for listener in load_balancer.listeners
             if listener.provisioning_status != constants.PENDING_DELETE and
             listener.provisioning_status != constants.DELETED]
+        # As the peer port will hold the tcp connection for keepalived and
+        # haproxy session synchronization, so here the security group rule
+        # should be just related with tcp protocol only.
         peer_ports = [
-            listener.peer_port for listener in load_balancer.listeners
+            (listener.peer_port,
+             constants.PROTOCOL_TCP.lower())
+            for listener in load_balancer.listeners
             if listener.provisioning_status != constants.PENDING_DELETE and
             listener.provisioning_status != constants.DELETED]
         updated_ports.extend(peer_ports)
         # Just going to use port_range_max for now because we can assume that
         # port_range_max and min will be the same since this driver is
         # responsible for creating these rules
-        old_ports = [rule.get('port_range_max')
+        old_ports = [(rule.get('port_range_max'), rule.get('protocol'))
                      for rule in rules.get('security_group_rules', [])
                      # Don't remove egress rules and don't
                      # confuse other protocols with None ports
                      # with the egress rules.  VRRP uses protocol
                      # 51 and 112
                      if rule.get('direction') != 'egress' and
-                     rule.get('protocol', '').lower() == 'tcp']
+                     rule.get('protocol', '').lower() in ['tcp', 'udp']]
         add_ports = set(updated_ports) - set(old_ports)
         del_ports = set(old_ports) - set(updated_ports)
         for rule in rules.get('security_group_rules', []):
-            if rule.get('port_range_max') in del_ports:
+            if (rule.get('protocol', '') and
+                    rule.get('protocol', '').lower() in ['tcp', 'udp'] and
+                    (rule.get('port_range_max'),
+                     rule.get('protocol')) in del_ports):
                 rule_id = rule.get('id')
                 try:
                     self.neutron_client.delete_security_group_rule(rule_id)
@@ -165,9 +197,10 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                              "it is already deleted.", rule_id)
 
         ethertype = self._get_ethertype_for_ip(load_balancer.vip.ip_address)
-        for port in add_ports:
-            self._create_security_group_rule(sec_grp_id, 'TCP', port_min=port,
-                                             port_max=port,
+        for port_protocol in add_ports:
+            self._create_security_group_rule(sec_grp_id, port_protocol[1],
+                                             port_min=port_protocol[0],
+                                             port_max=port_protocol[0],
                                              ethertype=ethertype)
 
         # Currently we are using the VIP network for VRRP
@@ -243,43 +276,65 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         LOG.exception(message)
         raise base.DeallocateVIPException(message)
 
-    @staticmethod
-    def _filter_amphora(amp):
-        return amp.status == constants.AMPHORA_ALLOCATED
-
     def _delete_security_group(self, vip, port):
         if self.sec_grp_enabled:
             sec_grp = self._get_lb_security_group(vip.load_balancer.id)
             if sec_grp:
-                sec_grp = sec_grp.get('id')
+                sec_grp_id = sec_grp.get('id')
                 LOG.info(
                     "Removing security group %(sg)s from port %(port)s",
-                    {'sg': sec_grp, 'port': vip.port_id})
+                    {'sg': sec_grp_id, 'port': vip.port_id})
                 raw_port = None
                 try:
                     if port:
                         raw_port = self.neutron_client.show_port(port.id)
                 except Exception:
                     LOG.warning('Unable to get port information for port '
-                                ' %s. Continuing to delete the security '
+                                '%s. Continuing to delete the security '
                                 'group.', port.id)
                 if raw_port:
                     sec_grps = raw_port.get(
                         'port', {}).get('security_groups', [])
-                    if sec_grp in sec_grps:
-                        sec_grps.remove(sec_grp)
-                    port_update = {'port': {'security_groups': sec_grps}}
-                    self.neutron_client.update_port(port.id, port_update)
+                    if sec_grp_id in sec_grps:
+                        sec_grps.remove(sec_grp_id)
+                        port_update = {'port': {'security_groups': sec_grps}}
+                        try:
+                            self.neutron_client.update_port(port.id,
+                                                            port_update)
+                        except neutron_client_exceptions.PortNotFoundClient:
+                            LOG.warning('Unable to update port information '
+                                        'for port %s. Continuing to delete '
+                                        'the security group since port not '
+                                        'found', port.id)
 
-                self._delete_vip_security_group(sec_grp)
+                try:
+                    self._delete_vip_security_group(sec_grp_id)
+                except base.DeallocateVIPException:
+                    # Try to delete any leftover ports on this security group.
+                    # Because this security group is created and managed by us,
+                    # it *should* only return ports that we own / can delete.
+                    LOG.warning('Failed to delete security group on first '
+                                'pass: %s', sec_grp_id)
+                    extra_ports = self._get_ports_by_security_group(sec_grp_id)
+                    for extra_port in extra_ports:
+                        port_id = extra_port.get('id')
+                        try:
+                            LOG.warning('Deleting extra port %s on security '
+                                        'group %s...', port_id, sec_grp_id)
+                            self.neutron_client.delete_port(port_id)
+                        except Exception:
+                            LOG.warning('Failed to delete extra port %s on '
+                                        'security group %s.',
+                                        port_id, sec_grp_id)
+                    # Now try it again
+                    self._delete_vip_security_group(sec_grp_id)
 
     def deallocate_vip(self, vip):
         """Delete the vrrp_port (instance port) in case nova didn't
 
         This can happen if a failover has occurred.
         """
-        for amphora in six.moves.filter(self._filter_amphora,
-                                        vip.load_balancer.amphorae):
+        for amphora in vip.load_balancer.amphorae:
             try:
                 self.neutron_client.delete_port(amphora.vrrp_port_id)
             except (neutron_client_exceptions.NotFound,
@@ -359,13 +414,20 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             fixed_ip['subnet_id'] = load_balancer.vip.subnet_id
         if load_balancer.vip.ip_address:
             fixed_ip['ip_address'] = load_balancer.vip.ip_address
+
+        # Make sure we are backward compatible with older neutron
+        if self._check_extension_enabled(PROJECT_ID_ALIAS):
+            project_id_key = 'project_id'
+        else:
+            project_id_key = 'tenant_id'
+
         # It can be assumed that network_id exists
         port = {'port': {'name': 'octavia-lb-' + load_balancer.id,
                          'network_id': load_balancer.vip.network_id,
                          'admin_state_up': False,
                          'device_id': 'lb-{0}'.format(load_balancer.id),
                          'device_owner': OCTAVIA_OWNER,
-                         'project_id': load_balancer.project_id}}
+                         project_id_key: load_balancer.project_id}}
 
         if fixed_ip:
             port['port']['fixed_ips'] = [fixed_ip]
@@ -379,7 +441,8 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             raise base.AllocateVIPException(
                 message,
                 orig_msg=getattr(e, 'message', None),
-                orig_code=getattr(e, 'status_code', None))
+                orig_code=getattr(e, 'status_code', None),
+            )
         new_port = utils.convert_port_dict_to_model(new_port)
         return self._port_to_vip(new_port, load_balancer)
 
@@ -439,12 +502,12 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                 server=compute_id, net_id=network_id, fixed_ip=ip_address,
                 port_id=None)
         except nova_client_exceptions.NotFound as e:
-            if 'Instance' in e.message:
-                raise base.AmphoraNotFound(e.message)
-            elif 'Network' in e.message:
-                raise base.NetworkNotFound(e.message)
+            if 'Instance' in str(e):
+                raise base.AmphoraNotFound(str(e))
+            elif 'Network' in str(e):
+                raise base.NetworkNotFound(str(e))
             else:
-                raise base.PlugNetworkException(e.message)
+                raise base.PlugNetworkException(str(e))
         except Exception:
             message = _('Error plugging amphora (compute_id: {compute_id}) '
                         'into network {network_id}.').format(
@@ -474,9 +537,17 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                             'Skipping.'.format(port_id=unplugger.port_id,
                                                compute_id=compute_id))
 
-    def update_vip(self, load_balancer):
+    def update_vip(self, load_balancer, for_delete=False):
         sec_grp = self._get_lb_security_group(load_balancer.id)
-        self._update_security_group_rules(load_balancer, sec_grp.get('id'))
+        if sec_grp:
+            self._update_security_group_rules(load_balancer, sec_grp.get('id'))
+        elif not for_delete:
+            raise exceptions.MissingVIPSecurityGroup(lb_id=load_balancer.id)
+        else:
+            LOG.warning('VIP security group missing when updating the VIP for '
+                        'delete on load balancer: {lb_id}. Skipping update '
+                        'because this is for delete.'.format(
+                            lb_id=load_balancer.id))
 
     def failover_preparation(self, amphora):
         if self.dns_integration_enabled:
@@ -513,12 +584,12 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             plugged_interface = self._nova_interface_to_octavia_interface(
                 amphora.compute_id, interface)
         except nova_client_exceptions.NotFound as e:
-            if 'Instance' in e.message:
-                raise base.AmphoraNotFound(e.message)
-            elif 'Network' in e.message:
-                raise base.NetworkNotFound(e.message)
+            if 'Instance' in str(e):
+                raise base.AmphoraNotFound(str(e))
+            elif 'Network' in str(e):
+                raise base.NetworkNotFound(str(e))
             else:
-                raise base.PlugNetworkException(e.message)
+                raise base.PlugNetworkException(str(e))
         except nova_client_exceptions.Conflict:
             LOG.info('Port %(portid)s is already plugged, '
                      'skipping', {'portid': port.id})
