@@ -21,6 +21,7 @@ from oslo_utils import uuidutils
 from octavia.common import constants
 import octavia.common.context
 from octavia.common import data_models
+from octavia.common import exceptions
 from octavia.db import repositories
 from octavia.tests.functional.api.v2 import base
 
@@ -54,6 +55,30 @@ class TestHealthMonitor(base.BaseAPITest):
             self.pool_with_listener.get('pool').get('id'))
         self.set_lb_status(self.lb_id)
         self.pool_repo = repositories.PoolRepository()
+        self._setup_udp_lb_resources()
+
+    def _setup_udp_lb_resources(self):
+        self.udp_lb = self.create_load_balancer(uuidutils.generate_uuid()).get(
+            'loadbalancer')
+        self.udp_lb_id = self.udp_lb.get('id')
+        self.set_lb_status(self.udp_lb_id)
+
+        self.udp_listener = self.create_listener(
+            constants.PROTOCOL_UDP, 8888,
+            self.udp_lb_id).get('listener')
+        self.udp_listener_id = self.udp_listener.get('id')
+        self.set_lb_status(self.udp_lb_id)
+
+        self.udp_pool_with_listener = self.create_pool(
+            None, constants.PROTOCOL_UDP, constants.LB_ALGORITHM_ROUND_ROBIN,
+            listener_id=self.udp_listener_id)
+        self.udp_pool_with_listener_id = (
+            self.udp_pool_with_listener.get('pool').get('id'))
+        self.set_lb_status(self.udp_lb_id)
+        self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
+        self.conf.config(
+            group='api_settings',
+            udp_connect_min_interval_health_monitor='3')
 
     def test_get(self):
         api_hm = self.create_health_monitor(
@@ -128,19 +153,15 @@ class TestHealthMonitor(base.BaseAPITest):
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
         self.assertEqual(self.NOT_AUTHORIZED_BODY, response.json)
 
-    def test_get_hides_deleted(self):
+    def test_get_deleted_gives_404(self):
         api_hm = self.create_health_monitor(
             self.pool_id, constants.HEALTH_MONITOR_HTTP,
             1, 1, 1, 1).get(self.root_tag)
 
-        response = self.get(self.HMS_PATH)
-        objects = response.json.get(self.root_tag_list)
-        self.assertEqual(len(objects), 1)
         self.set_object_status(self.health_monitor_repo, api_hm.get('id'),
                                provisioning_status=constants.DELETED)
-        response = self.get(self.HMS_PATH)
-        objects = response.json.get(self.root_tag_list)
-        self.assertEqual(len(objects), 0)
+        self.get(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+                 status=404)
 
     def test_bad_get(self):
         self.get(self.HM_PATH.format(
@@ -325,6 +346,20 @@ class TestHealthMonitor(base.BaseAPITest):
         self.assertIn((hm2.get('id'), hm2.get('type')), hm_id_protocols)
         self.assertIn((hm3.get('id'), hm3.get('type')), hm_id_protocols)
 
+    def test_get_all_hides_deleted(self):
+        api_hm = self.create_health_monitor(
+            self.pool_id, constants.HEALTH_MONITOR_HTTP,
+            1, 1, 1, 1).get(self.root_tag)
+
+        response = self.get(self.HMS_PATH)
+        objects = response.json.get(self.root_tag_list)
+        self.assertEqual(len(objects), 1)
+        self.set_object_status(self.health_monitor_repo, api_hm.get('id'),
+                               provisioning_status=constants.DELETED)
+        response = self.get(self.HMS_PATH)
+        objects = response.json.get(self.root_tag_list)
+        self.assertEqual(len(objects), 0)
+
     def test_get_by_project_id(self):
         project1_id = uuidutils.generate_uuid()
         project2_id = uuidutils.generate_uuid()
@@ -631,6 +666,160 @@ class TestHealthMonitor(base.BaseAPITest):
         self.assertEqual('/', api_hm.get('url_path'))
         self.assertEqual('200', api_hm.get('expected_codes'))
 
+    def test_create_http_full(self):
+        api_hm = self.create_health_monitor(
+            self.pool_id, constants.HEALTH_MONITOR_HTTP,
+            1, 1, 1, 1, admin_state_up=False, expected_codes='200',
+            http_method='GET', name='Test HM', url_path='/').get(self.root_tag)
+        self.assertEqual(constants.HEALTH_MONITOR_HTTP, api_hm.get('type'))
+        self.assertEqual(1, api_hm.get('delay'))
+        self.assertEqual(1, api_hm.get('timeout'))
+        self.assertEqual(1, api_hm.get('max_retries_down'))
+        self.assertEqual(1, api_hm.get('max_retries'))
+        self.assertFalse(api_hm.get('admin_state_up'))
+        self.assertEqual('Test HM', api_hm.get('name'))
+        self.assertEqual('GET', api_hm.get('http_method'))
+        self.assertEqual('/', api_hm.get('url_path'))
+        self.assertEqual('200', api_hm.get('expected_codes'))
+
+    def test_create_udp_case(self):
+        api_hm = self.create_health_monitor(
+            self.udp_pool_with_listener_id,
+            constants.HEALTH_MONITOR_UDP_CONNECT,
+            3, 1, 1, 1).get(self.root_tag)
+        self.assert_correct_status(
+            lb_id=self.udp_lb_id, listener_id=self.udp_listener_id,
+            pool_id=self.udp_pool_with_listener_id, hm_id=api_hm.get('id'),
+            lb_prov_status=constants.PENDING_UPDATE,
+            listener_prov_status=constants.PENDING_UPDATE,
+            pool_prov_status=constants.PENDING_UPDATE,
+            hm_prov_status=constants.PENDING_CREATE,
+            hm_op_status=constants.OFFLINE)
+        self.set_lb_status(self.udp_lb_id)
+        self.assertEqual(constants.HEALTH_MONITOR_UDP_CONNECT,
+                         api_hm.get('type'))
+        self.assertEqual(3, api_hm.get('delay'))
+        self.assertEqual(1, api_hm.get('timeout'))
+        self.assertEqual(1, api_hm.get('max_retries_down'))
+        self.assertEqual(1, api_hm.get('max_retries'))
+        # Verify the L7 fields is None
+        self.assertIsNone(api_hm.get('http_method'))
+        self.assertIsNone(api_hm.get('url_path'))
+        self.assertIsNone(api_hm.get('expected_codes'))
+
+    def test_udp_case_when_udp_connect_min_interval_health_monitor_set(self):
+        # negative case first
+        req_dict = {'pool_id': self.udp_pool_with_listener_id,
+                    'type': constants.HEALTH_MONITOR_UDP_CONNECT,
+                    'delay': 1,
+                    'timeout': 1,
+                    'max_retries_down': 1,
+                    'max_retries': 1}
+        res = self.post(self.HMS_PATH, self._build_body(req_dict), status=400,
+                        expect_errors=True)
+        expect_error_msg = ("Validation failure: The request delay value 1 "
+                            "should be larger than 3 for %s health monitor "
+                            "type.") % constants.HEALTH_MONITOR_UDP_CONNECT
+
+        self.assertEqual(expect_error_msg, res.json['faultstring'])
+        self.assert_correct_status(
+            lb_id=self.udp_lb_id, listener_id=self.udp_listener_id,
+            pool_id=self.udp_pool_with_listener_id)
+
+        # pass cases
+        self.conf.config(
+            group='api_settings',
+            udp_connect_min_interval_health_monitor='-3')
+        res = self.post(self.HMS_PATH, self._build_body(req_dict))
+        self.assert_correct_status(
+            lb_id=self.udp_lb_id, listener_id=self.udp_listener_id,
+            pool_id=self.udp_pool_with_listener_id,
+            hm_id=res.json['healthmonitor']['id'],
+            lb_prov_status=constants.PENDING_UPDATE,
+            listener_prov_status=constants.PENDING_UPDATE,
+            pool_prov_status=constants.PENDING_UPDATE,
+            hm_prov_status=constants.PENDING_CREATE,
+            hm_op_status=constants.OFFLINE)
+
+    def test_negative_create_udp_case(self):
+        req_dict = {'pool_id': self.udp_pool_with_listener_id,
+                    'delay': 3,
+                    'timeout': 1,
+                    'max_retries_down': 1,
+                    'max_retries': 1}
+        expect_error_msg = ("Validation failure: The associated pool protocol "
+                            "is %(pool_protocol)s, so only a %(type)s health "
+                            "monitor is supported.") % {
+            'pool_protocol': constants.PROTOCOL_UDP,
+            'type': constants.HEALTH_MONITOR_UDP_CONNECT}
+
+        # Not allowed types, url_path, expected_codes specified.
+        update_req = {'type': constants.HEALTH_MONITOR_TCP}
+        req_dict.update(update_req)
+        res = self.post(self.HMS_PATH, self._build_body(req_dict), status=400,
+                        expect_errors=True)
+        self.assertEqual(expect_error_msg, res.json['faultstring'])
+        self.assert_correct_status(
+            lb_id=self.udp_lb_id, listener_id=self.udp_listener_id,
+            pool_id=self.udp_pool_with_listener_id)
+
+        update_req = {'type': constants.HEALTH_MONITOR_UDP_CONNECT}
+        req_dict.update(update_req)
+        for req in [{'http_method':
+                     constants.HEALTH_MONITOR_HTTP_METHOD_GET},
+                    {'url_path': constants.HEALTH_MONITOR_DEFAULT_URL_PATH},
+                    {'expected_codes':
+                        constants.HEALTH_MONITOR_DEFAULT_EXPECTED_CODES}]:
+            req_dict.update(req)
+            res = self.post(self.HMS_PATH, self._build_body(req_dict),
+                            status=400,
+                            expect_errors=True)
+            self.assertEqual(expect_error_msg, res.json['faultstring'])
+            self.assert_correct_status(
+                lb_id=self.udp_lb_id, listener_id=self.udp_listener_id,
+                pool_id=self.udp_pool_with_listener_id)
+
+        # Hit error during create with a non-UDP pool
+        req_dict = {'pool_id': self.pool_with_listener_id,
+                    'delay': 1,
+                    'timeout': 1,
+                    'max_retries_down': 1,
+                    'max_retries': 1,
+                    'type': constants.HEALTH_MONITOR_UDP_CONNECT}
+        expect_error_msg = ("Validation failure: The %(type)s type is only "
+                            "supported for pools of type "
+                            "%(protocol)s.") % {
+            'type': constants.HEALTH_MONITOR_UDP_CONNECT,
+            'protocol': constants.PROTOCOL_UDP}
+        res = self.post(self.HMS_PATH, self._build_body(req_dict),
+                        status=400,
+                        expect_errors=True)
+        self.assertEqual(expect_error_msg, res.json['faultstring'])
+        self.assert_correct_status(
+            lb_id=self.udp_lb_id, listener_id=self.udp_listener_id,
+            pool_id=self.udp_pool_with_listener_id)
+
+    def test_ensure_L7_fields_filled_during_create(self):
+        # Create a health monitor with a load balancer pool
+        api_hm = self.create_health_monitor(
+            self.pool_id,
+            constants.PROTOCOL_HTTP,
+            1, 1, 1, 1).get(self.root_tag)
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=self.listener_id,
+            pool_id=self.pool_id, hm_id=api_hm.get('id'),
+            lb_prov_status=constants.PENDING_UPDATE,
+            pool_prov_status=constants.PENDING_UPDATE,
+            hm_prov_status=constants.PENDING_CREATE,
+            hm_op_status=constants.OFFLINE)
+        self.set_lb_status(self.lb_id)
+        self.assertEqual(constants.HEALTH_MONITOR_HTTP_DEFAULT_METHOD,
+                         api_hm.get('http_method'))
+        self.assertEqual(constants.HEALTH_MONITOR_DEFAULT_URL_PATH,
+                         api_hm.get('url_path'))
+        self.assertEqual(constants.HEALTH_MONITOR_DEFAULT_EXPECTED_CODES,
+                         api_hm.get('expected_codes'))
+
     def test_create_authorized(self):
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
@@ -771,20 +960,140 @@ class TestHealthMonitor(base.BaseAPITest):
             lb_id=self.lb_id, listener_id=self.listener_id,
             pool_id=self.pool_id)
 
-    def test_create_with_bad_handler(self):
-        self.handler_mock().health_monitor.create.side_effect = Exception()
-        api_hm = self.create_health_monitor(
-            self.pool_with_listener_id,
-            constants.HEALTH_MONITOR_HTTP, 1, 1, 1, 1).get(self.root_tag)
+    def test_create_ping_when_ping_disabled(self):
+        self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
+        self.conf.config(group='api_settings',
+                         allow_ping_health_monitors=False)
+        req_dict = {'pool_id': self.pool_id,
+                    'type': constants.HEALTH_MONITOR_PING,
+                    'delay': 1,
+                    'timeout': 1,
+                    'max_retries_down': 1,
+                    'max_retries': 1}
+        self.post(self.HMS_PATH, self._build_body(req_dict), status=400)
         self.assert_correct_status(
             lb_id=self.lb_id, listener_id=self.listener_id,
-            pool_id=self.pool_with_listener_id,
-            hm_id=api_hm.get('id'),
-            lb_prov_status=constants.ACTIVE,
-            listener_prov_status=constants.ACTIVE,
-            pool_prov_status=constants.ACTIVE,
-            hm_prov_status=constants.ERROR,
-            hm_op_status=constants.OFFLINE)
+            pool_id=self.pool_id)
+
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_create_with_bad_provider(self, mock_provider):
+        mock_provider.side_effect = exceptions.ProviderDriverError(
+            prov='bad_driver', user_msg='broken')
+        req_dict = {'pool_id': self.pool_id,
+                    'type': constants.HEALTH_MONITOR_HTTP,
+                    'delay': 1,
+                    'timeout': 1,
+                    'max_retries_down': 1,
+                    'max_retries': 1,
+                    'url_path': '/foo'}
+        response = self.post(self.HMS_PATH, self._build_body(req_dict),
+                             status=500)
+        self.assertIn('Provider \'bad_driver\' reports error: broken',
+                      response.json.get('faultstring'))
+
+    def test_create_with_type_none(self):
+        req_dict = {'pool_id': self.pool_id,
+                    'type': None,
+                    'delay': 1,
+                    'timeout': 1,
+                    'max_retries_down': 1,
+                    'max_retries': 1,
+                    'url_path': '/'}
+        self.post(self.HMS_PATH, self._build_body(req_dict), status=400)
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=self.listener_id,
+            pool_id=self.pool_id)
+
+    def test_create_with_delay_none(self):
+        req_dict = {'pool_id': self.pool_id,
+                    'type': constants.HEALTH_MONITOR_HTTP,
+                    'delay': None,
+                    'timeout': 1,
+                    'max_retries_down': 1,
+                    'max_retries': 1,
+                    'url_path': '/'}
+        self.post(self.HMS_PATH, self._build_body(req_dict), status=400)
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=self.listener_id,
+            pool_id=self.pool_id)
+
+    def test_create_with_max_retries_none(self):
+        req_dict = {'pool_id': self.pool_id,
+                    'type': constants.HEALTH_MONITOR_HTTP,
+                    'delay': 1,
+                    'timeout': 1,
+                    'max_retries_down': 1,
+                    'max_retries': None,
+                    'url_path': '/'}
+        self.post(self.HMS_PATH, self._build_body(req_dict), status=400)
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=self.listener_id,
+            pool_id=self.pool_id)
+
+    def test_create_with_timeout_none(self):
+        req_dict = {'pool_id': self.pool_id,
+                    'type': constants.HEALTH_MONITOR_HTTP,
+                    'delay': 1,
+                    'timeout': None,
+                    'max_retries_down': 1,
+                    'max_retries': 1,
+                    'url_path': '/'}
+        self.post(self.HMS_PATH, self._build_body(req_dict), status=400)
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=self.listener_id,
+            pool_id=self.pool_id)
+
+    def test_create_with_pool_id_none(self):
+        req_dict = {'pool_id': None,
+                    'type': constants.HEALTH_MONITOR_HTTP,
+                    'delay': 1,
+                    'timeout': 1,
+                    'max_retries_down': 1,
+                    'max_retries': 1,
+                    'url_path': '/'}
+        self.post(self.HMS_PATH, self._build_body(req_dict), status=404)
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=self.listener_id,
+            pool_id=self.pool_id)
+
+    def test_create_TCP_with_http_method(self):
+        req_dict = {'pool_id': self.pool_id,
+                    'type': constants.HEALTH_MONITOR_TCP,
+                    'delay': 1,
+                    'timeout': 1,
+                    'max_retries_down': 1,
+                    'max_retries': 1,
+                    'http_method': constants.HEALTH_MONITOR_HTTP_METHOD_GET}
+        self.post(self.HMS_PATH, self._build_body(req_dict), status=400)
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=self.listener_id,
+            pool_id=self.pool_id)
+
+    def test_create_TCP_with_url_path(self):
+        req_dict = {'pool_id': self.pool_id,
+                    'type': constants.HEALTH_MONITOR_TCP,
+                    'delay': 1,
+                    'timeout': 1,
+                    'max_retries_down': 1,
+                    'max_retries': 1,
+                    'url_path': '/'}
+        self.post(self.HMS_PATH, self._build_body(req_dict), status=400)
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=self.listener_id,
+            pool_id=self.pool_id)
+
+    def test_create_TCP_with_expected_codes(self):
+        req_dict = {'pool_id': self.pool_id,
+                    'type': constants.HEALTH_MONITOR_TCP,
+                    'delay': 1,
+                    'timeout': 1,
+                    'max_retries_down': 1,
+                    'max_retries': 1,
+                    'expected_codes': '200'}
+        self.post(self.HMS_PATH, self._build_body(req_dict), status=400)
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=self.listener_id,
+            pool_id=self.pool_id)
 
     def test_duplicate_create(self):
         self.create_health_monitor(
@@ -820,6 +1129,29 @@ class TestHealthMonitor(base.BaseAPITest):
             listener_prov_status=constants.PENDING_UPDATE,
             pool_prov_status=constants.PENDING_UPDATE,
             hm_prov_status=constants.PENDING_UPDATE)
+        response = self.get(self.HM_PATH.format(
+            healthmonitor_id=api_hm.get('id'))).json.get(self.root_tag)
+        self.assertEqual(2, response[constants.MAX_RETRIES])
+
+    def test_update_TCP(self):
+        api_hm = self.create_health_monitor(
+            self.pool_with_listener_id,
+            constants.HEALTH_MONITOR_TCP, 1, 1, 1, 1).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_hm = {'max_retries': 2}
+        self.put(
+            self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+            self._build_body(new_hm))
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=self.listener_id,
+            pool_id=self.pool_with_listener_id, hm_id=api_hm.get('id'),
+            lb_prov_status=constants.PENDING_UPDATE,
+            listener_prov_status=constants.PENDING_UPDATE,
+            pool_prov_status=constants.PENDING_UPDATE,
+            hm_prov_status=constants.PENDING_UPDATE)
+        response = self.get(self.HM_PATH.format(
+            healthmonitor_id=api_hm.get('id'))).json.get(self.root_tag)
+        self.assertEqual(2, response[constants.MAX_RETRIES])
 
     def test_update_authorized(self):
         api_hm = self.create_health_monitor(
@@ -887,6 +1219,61 @@ class TestHealthMonitor(base.BaseAPITest):
             pool_prov_status=constants.ACTIVE,
             hm_prov_status=constants.ACTIVE)
 
+    def test_update_udp_case(self):
+        api_hm = self.create_health_monitor(
+            self.udp_pool_with_listener_id,
+            constants.HEALTH_MONITOR_UDP_CONNECT, 3, 1, 1, 1).get(
+            self.root_tag)
+        self.set_lb_status(self.udp_lb_id)
+        new_hm = {'max_retries': 2}
+        self.put(
+            self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+            self._build_body(new_hm))
+        self.assert_correct_status(
+            lb_id=self.udp_lb_id, listener_id=self.udp_listener_id,
+            pool_id=self.udp_pool_with_listener_id, hm_id=api_hm.get('id'),
+            lb_prov_status=constants.PENDING_UPDATE,
+            listener_prov_status=constants.PENDING_UPDATE,
+            pool_prov_status=constants.PENDING_UPDATE,
+            hm_prov_status=constants.PENDING_UPDATE)
+
+    def test_negative_update_udp_case(self):
+        api_hm = self.create_health_monitor(
+            self.udp_pool_with_listener_id,
+            constants.HEALTH_MONITOR_UDP_CONNECT, 3, 1, 1, 1).get(
+            self.root_tag)
+        self.set_lb_status(self.udp_lb_id)
+
+        # Hit error during update with invalid parameter
+        req_dict = {'delay': 3,
+                    'timeout': 1,
+                    'max_retries_down': 1,
+                    'max_retries': 1,
+                    'http_method': constants.HEALTH_MONITOR_HTTP_METHOD_GET}
+        expect_error_msg = (("http_method is not a valid option for health "
+                             "monitors of type %s") %
+                            constants.HEALTH_MONITOR_UDP_CONNECT)
+        res = self.put(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+                       self._build_body(req_dict),
+                       status=400, expect_errors=True)
+        self.assertEqual(expect_error_msg, res.json['faultstring'])
+        self.assert_correct_status(
+            lb_id=self.udp_lb_id, listener_id=self.udp_listener_id,
+            pool_id=self.udp_pool_with_listener_id)
+
+        # Hit error during update with smaller delay value
+        req_dict = {'delay': 1}
+        expect_error_msg = ("Validation failure: The request delay value 1 "
+                            "should be larger than 3 for %s health monitor "
+                            "type.") % constants.HEALTH_MONITOR_UDP_CONNECT
+        res = self.put(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+                       self._build_body(req_dict),
+                       status=400, expect_errors=True)
+        self.assertEqual(expect_error_msg, res.json['faultstring'])
+        self.assert_correct_status(
+            lb_id=self.udp_lb_id, listener_id=self.udp_listener_id,
+            pool_id=self.udp_pool_with_listener_id)
+
     def test_bad_update(self):
         api_hm = self.create_health_monitor(self.pool_with_listener_id,
                                             constants.HEALTH_MONITOR_HTTP,
@@ -896,19 +1283,92 @@ class TestHealthMonitor(base.BaseAPITest):
         self.put(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
                  self._build_body(new_hm), status=400)
 
-    def test_update_with_bad_handler(self):
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_update_with_bad_provider(self, mock_provider):
         api_hm = self.create_health_monitor(
             self.pool_with_listener_id,
             constants.HEALTH_MONITOR_HTTP, 1, 1, 1, 1).get(self.root_tag)
         self.set_lb_status(self.lb_id)
         new_hm = {'max_retries': 2}
-        self.handler_mock().health_monitor.update.side_effect = Exception()
-        self.put(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
-                 self._build_body(new_hm))
-        self.assert_correct_status(
-            lb_id=self.lb_id, listener_id=self.listener_id,
-            pool_id=self.pool_with_listener_id, hm_id=api_hm.get('id'),
-            hm_prov_status=constants.ERROR)
+        mock_provider.side_effect = exceptions.ProviderDriverError(
+            prov='bad_driver', user_msg='broken')
+        response = self.put(
+            self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+            self._build_body(new_hm), status=500)
+        self.assertIn('Provider \'bad_driver\' reports error: broken',
+                      response.json.get('faultstring'))
+
+    def test_update_TCP_setting_http_method(self):
+        api_hm = self.create_health_monitor(
+            self.pool_with_listener_id,
+            constants.HEALTH_MONITOR_TCP, 1, 1, 1, 1).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_hm = {'http_method': constants.HEALTH_MONITOR_HTTP_METHOD_GET}
+        self.put(
+            self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+            self._build_body(new_hm), status=400)
+
+    def test_update_TCP_setting_url_path(self):
+        api_hm = self.create_health_monitor(
+            self.pool_with_listener_id,
+            constants.HEALTH_MONITOR_TCP, 1, 1, 1, 1).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_hm = {'url_path': '/'}
+        self.put(
+            self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+            self._build_body(new_hm), status=400)
+
+    def test_update_TCP_setting_expected_codes(self):
+        api_hm = self.create_health_monitor(
+            self.pool_with_listener_id,
+            constants.HEALTH_MONITOR_TCP, 1, 1, 1, 1).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_hm = {'expected_codes': '200'}
+        self.put(
+            self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+            self._build_body(new_hm), status=400)
+
+    def test_update_HTTP_http_method_none(self):
+        api_hm = self.create_health_monitor(
+            self.pool_with_listener_id,
+            constants.HEALTH_MONITOR_HTTP, 1, 1, 1, 1).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_hm = {'http_method': None}
+        self.put(
+            self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+            self._build_body(new_hm))
+        response = self.get(self.HM_PATH.format(
+            healthmonitor_id=api_hm.get('id'))).json.get(self.root_tag)
+        self.assertEqual(constants.HEALTH_MONITOR_HTTP_METHOD_GET,
+                         response['http_method'])
+
+    def test_update_HTTP_url_path_none(self):
+        api_hm = self.create_health_monitor(
+            self.pool_with_listener_id,
+            constants.HEALTH_MONITOR_HTTP, 1, 1, 1, 1).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_hm = {'url_path': None}
+        self.put(
+            self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+            self._build_body(new_hm))
+        response = self.get(self.HM_PATH.format(
+            healthmonitor_id=api_hm.get('id'))).json.get(self.root_tag)
+        self.assertEqual(constants.HEALTH_MONITOR_DEFAULT_URL_PATH,
+                         response['url_path'])
+
+    def test_update_HTTP_expected_codes_none(self):
+        api_hm = self.create_health_monitor(
+            self.pool_with_listener_id,
+            constants.HEALTH_MONITOR_HTTP, 1, 1, 1, 1).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_hm = {'expected_codes': None}
+        self.put(
+            self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+            self._build_body(new_hm))
+        response = self.get(self.HM_PATH.format(
+            healthmonitor_id=api_hm.get('id'))).json.get(self.root_tag)
+        self.assertEqual(constants.HEALTH_MONITOR_DEFAULT_EXPECTED_CODES,
+                         response['expected_codes'])
 
     def test_delete(self):
         api_hm = self.create_health_monitor(
@@ -1011,7 +1471,8 @@ class TestHealthMonitor(base.BaseAPITest):
             self.HM_PATH.format(healthmonitor_id=uuidutils.generate_uuid()),
             status=404)
 
-    def test_delete_with_bad_handler(self):
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_delete_with_bad_provider(self, mock_provider):
         api_hm = self.create_health_monitor(
             self.pool_with_listener_id,
             constants.HEALTH_MONITOR_HTTP, 1, 1, 1, 1).get(self.root_tag)
@@ -1023,12 +1484,11 @@ class TestHealthMonitor(base.BaseAPITest):
         self.assertIsNone(api_hm.pop('updated_at'))
         self.assertIsNotNone(hm.pop('updated_at'))
         self.assertEqual(api_hm, hm)
-        self.handler_mock().health_monitor.delete.side_effect = Exception()
-        self.delete(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')))
-        self.assert_correct_status(
-            lb_id=self.lb_id, listener_id=self.listener_id,
-            pool_id=self.pool_with_listener_id, hm_id=api_hm.get('id'),
-            hm_prov_status=constants.ERROR)
+
+        mock_provider.side_effect = exceptions.ProviderDriverError(
+            prov='bad_driver', user_msg='broken')
+        self.delete(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+                    status=500)
 
     def test_create_when_lb_pending_update(self):
         self.put(self.LB_PATH.format(lb_id=self.lb_id),
@@ -1076,6 +1536,15 @@ class TestHealthMonitor(base.BaseAPITest):
         self.put(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
                  body=self._build_body(new_hm), status=409)
 
+    def test_update_already_deleted(self):
+        api_hm = self.create_health_monitor(
+            self.pool_id, constants.HEALTH_MONITOR_HTTP,
+            1, 1, 1, 1).get(self.root_tag)
+        # This updates the child objects
+        self.set_lb_status(self.lb_id, status=constants.DELETED)
+        self.put(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+                 body=self._build_body({'max_retries': 2}), status=404)
+
     def test_delete_when_lb_pending_delete(self):
         api_hm = self.create_health_monitor(
             self.pool_id, constants.HEALTH_MONITOR_HTTP,
@@ -1093,4 +1562,4 @@ class TestHealthMonitor(base.BaseAPITest):
         # This updates the child objects
         self.set_lb_status(self.lb_id, status=constants.DELETED)
         self.delete(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
-                    status=204)
+                    status=404)
