@@ -23,6 +23,7 @@ from oslo_utils import uuidutils
 from octavia.common import constants
 import octavia.common.context
 from octavia.common import data_models
+from octavia.common import exceptions
 from octavia.tests.functional.api.v2 import base
 
 
@@ -364,6 +365,19 @@ class TestListener(base.BaseAPITest):
         self.assertEqual(li1['id'],
                          lis['listeners'][0]['id'])
 
+    def test_get_all_hides_deleted(self):
+        api_listener = self.create_listener(
+            constants.PROTOCOL_HTTP, 80, self.lb_id).get(self.root_tag)
+
+        response = self.get(self.LISTENERS_PATH)
+        objects = response.json.get(self.root_tag_list)
+        self.assertEqual(len(objects), 1)
+        self.set_object_status(self.listener_repo, api_listener.get('id'),
+                               provisioning_status=constants.DELETED)
+        response = self.get(self.LISTENERS_PATH)
+        objects = response.json.get(self.root_tag_list)
+        self.assertEqual(len(objects), 0)
+
     def test_get(self):
         listener = self.create_listener(
             constants.PROTOCOL_HTTP, 80, self.lb_id).get(self.root_tag)
@@ -420,25 +434,27 @@ class TestListener(base.BaseAPITest):
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
         self.assertEqual(self.NOT_AUTHORIZED_BODY, response.json)
 
-    def test_get_hides_deleted(self):
+    def test_get_deleted_gives_404(self):
         api_listener = self.create_listener(
             constants.PROTOCOL_HTTP, 80, self.lb_id).get(self.root_tag)
 
-        response = self.get(self.LISTENERS_PATH)
-        objects = response.json.get(self.root_tag_list)
-        self.assertEqual(len(objects), 1)
         self.set_object_status(self.listener_repo, api_listener.get('id'),
                                provisioning_status=constants.DELETED)
-        response = self.get(self.LISTENERS_PATH)
-        objects = response.json.get(self.root_tag_list)
-        self.assertEqual(len(objects), 0)
+        self.get(self.LISTENER_PATH.format(listener_id=api_listener.get('id')),
+                 status=404)
 
     def test_get_bad_listener_id(self):
         listener_path = self.listener_path
         self.get(listener_path.format(listener_id='SEAN-CONNERY'), status=404)
 
-    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificate_data')
-    def test_create(self, mock_load_cert, **optionals):
+    # TODO(johnsom) Fix this when there is a noop certificate manager
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_create(self, mock_cert_data, response_status=201, **optionals):
+        cert1 = data_models.TLSContainer(certificate='cert 1')
+        cert2 = data_models.TLSContainer(certificate='cert 2')
+        cert3 = data_models.TLSContainer(certificate='cert 3')
+        mock_cert_data.return_value = {'tls_cert': cert1,
+                                       'sni_certs': [cert2, cert3]}
         sni1 = uuidutils.generate_uuid()
         sni2 = uuidutils.generate_uuid()
         lb_listener = {'name': 'listener1', 'default_pool_id': None,
@@ -453,7 +469,9 @@ class TestListener(base.BaseAPITest):
                        'loadbalancer_id': self.lb_id}
         lb_listener.update(optionals)
         body = self._build_body(lb_listener)
-        response = self.post(self.LISTENERS_PATH, body)
+        response = self.post(self.LISTENERS_PATH, body, status=response_status)
+        if response_status >= 300:
+            return response
         listener_api = response.json['listener']
         extra_expects = {'provisioning_status': constants.PENDING_CREATE,
                          'operating_status': constants.OFFLINE}
@@ -474,6 +492,106 @@ class TestListener(base.BaseAPITest):
         self.assert_correct_lb_status(self.lb_id, constants.ONLINE,
                                       constants.PENDING_UPDATE)
         self.assert_final_listener_statuses(self.lb_id, listener_api.get('id'))
+        return listener_api
+
+    def test_create_with_timeouts(self):
+        optionals = {
+            'timeout_client_data': 1,
+            'timeout_member_connect': 2,
+            'timeout_member_data': constants.MIN_TIMEOUT,
+            'timeout_tcp_inspect': constants.MAX_TIMEOUT,
+        }
+        listener_api = self.test_create(**optionals)
+        self.assertEqual(1, listener_api.get('timeout_client_data'))
+        self.assertEqual(2, listener_api.get('timeout_member_connect'))
+        self.assertEqual(constants.MIN_TIMEOUT,
+                         listener_api.get('timeout_member_data'))
+        self.assertEqual(constants.MAX_TIMEOUT,
+                         listener_api.get('timeout_tcp_inspect'))
+
+    def test_create_with_timeouts_too_high(self):
+        optionals = {
+            'timeout_client_data': 1,
+            'timeout_member_connect': 2,
+            'timeout_member_data': 3,
+            'timeout_tcp_inspect': constants.MAX_TIMEOUT + 1,
+        }
+        resp = self.test_create(response_status=400, **optionals).json
+        fault = resp.get('faultstring')
+        self.assertIn(
+            'Invalid input for field/attribute timeout_tcp_inspect', fault)
+        self.assertIn(
+            'Value should be lower or equal to {0}'.format(
+                constants.MAX_TIMEOUT), fault)
+
+    def test_create_with_timeouts_too_low(self):
+        optionals = {
+            'timeout_client_data': 1,
+            'timeout_member_connect': 2,
+            'timeout_member_data': 3,
+            'timeout_tcp_inspect': constants.MIN_TIMEOUT - 1,
+        }
+        resp = self.test_create(response_status=400, **optionals).json
+        fault = resp.get('faultstring')
+        self.assertIn(
+            'Invalid input for field/attribute timeout_tcp_inspect', fault)
+        self.assertIn(
+            'Value should be greater or equal to {0}'.format(
+                constants.MIN_TIMEOUT), fault)
+
+    def test_create_udp_case(self):
+        api_listener = self.create_listener(constants.PROTOCOL_UDP, 6666,
+                                            self.lb_id).get(self.root_tag)
+        self.assertEqual(constants.PROTOCOL_UDP, api_listener.get('protocol'))
+        self.assertEqual(6666, api_listener.get('protocol_port'))
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=api_listener.get('id'),
+            lb_prov_status=constants.PENDING_UPDATE,
+            listener_prov_status=constants.PENDING_CREATE,
+            listener_op_status=constants.OFFLINE)
+
+    def test_negative_create_udp_case(self):
+        sni1 = uuidutils.generate_uuid()
+        sni2 = uuidutils.generate_uuid()
+        req_dict = {'name': 'listener1', 'default_pool_id': None,
+                    'description': 'desc1',
+                    'admin_state_up': False,
+                    'protocol': constants.PROTOCOL_UDP,
+                    'protocol_port': 6666, 'connection_limit': 10,
+                    'default_tls_container_ref': uuidutils.generate_uuid(),
+                    'sni_container_refs': [sni1, sni2],
+                    'insert_headers': {
+                        "X-Forwarded-Port": "true",
+                        "X-Forwarded-For": "true"},
+                    'loadbalancer_id': self.lb_id}
+        expect_error_msg = (
+            "Validation failure: %s protocol listener does not support TLS or "
+            "header insertion.") % constants.PROTOCOL_UDP
+        res = self.post(self.LISTENERS_PATH, self._build_body(req_dict),
+                        status=400, expect_errors=True)
+        self.assertEqual(expect_error_msg, res.json['faultstring'])
+        self.assert_correct_status(lb_id=self.lb_id)
+
+        # Default pool protocol is udp which is different with listener
+        # protocol.
+        udp_pool_id = self.create_pool(
+            self.lb_id, constants.PROTOCOL_UDP,
+            constants.LB_ALGORITHM_ROUND_ROBIN).get('pool').get('id')
+        self.set_lb_status(self.lb_id)
+        lb_listener = {'name': 'listener1',
+                       'default_pool_id': udp_pool_id,
+                       'description': 'desc1',
+                       'admin_state_up': False,
+                       'protocol': constants.PROTOCOL_HTTP,
+                       'protocol_port': 80,
+                       'loadbalancer_id': self.lb_id}
+        expect_error_msg = ("Validation failure: Listeners of type %s can "
+                            "only have pools of "
+                            "type UDP.") % constants.PROTOCOL_UDP
+        res = self.post(self.LISTENERS_PATH, self._build_body(lb_listener),
+                        status=400, expect_errors=True)
+        self.assertEqual(expect_error_msg, res.json['faultstring'])
+        self.assert_correct_status(lb_id=self.lb_id)
 
     def test_create_duplicate_fails(self):
         self.create_listener(constants.PROTOCOL_HTTP, 80, self.lb_id)
@@ -481,8 +599,7 @@ class TestListener(base.BaseAPITest):
         self.create_listener(constants.PROTOCOL_HTTP, 80, self.lb_id,
                              status=409)
 
-    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificate_data')
-    def test_create_bad_tls_ref(self, mock_load_cert):
+    def test_create_bad_tls_ref(self):
         sni1 = uuidutils.generate_uuid()
         sni2 = uuidutils.generate_uuid()
         tls_ref = uuidutils.generate_uuid()
@@ -494,7 +611,7 @@ class TestListener(base.BaseAPITest):
                        'loadbalancer_id': self.lb_id}
 
         body = self._build_body(lb_listener)
-        mock_load_cert.side_effect = [
+        self.cert_manager_mock().get_cert.side_effect = [
             Exception("bad cert"), None, Exception("bad_cert")]
         response = self.post(self.LISTENERS_PATH, body, status=400).json
         self.assertIn(sni1, response['faultstring'])
@@ -588,19 +705,27 @@ class TestListener(base.BaseAPITest):
         body = self._build_body(lb_listener)
         self.post(self.LISTENERS_PATH, body, status=403)
 
-    def test_create_with_bad_handler(self):
-        self.handler_mock().listener.create.side_effect = Exception()
-        api_listener = self.create_listener(
-            constants.PROTOCOL_HTTP, 80,
-            self.lb_id).get(self.root_tag)
-        self.assert_correct_status(
-            lb_id=self.lb_id,
-            listener_id=api_listener.get('id'),
-            listener_prov_status=constants.ERROR,
-            listener_op_status=constants.OFFLINE)
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_create_with_bad_provider(self, mock_provider):
+        mock_provider.side_effect = exceptions.ProviderDriverError(
+            prov='bad_driver', user_msg='broken')
+        lb_listener = {'name': 'listener1',
+                       'protocol': constants.PROTOCOL_HTTP,
+                       'protocol_port': 80,
+                       'loadbalancer_id': self.lb_id}
+        body = self._build_body(lb_listener)
+        response = self.post(self.LISTENERS_PATH, body, status=500)
+        self.assertIn('Provider \'bad_driver\' reports error: broken',
+                      response.json.get('faultstring'))
 
-    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificate_data')
-    def test_create_authorized(self, mock_load_cert, **optionals):
+    # TODO(johnsom) Fix this when there is a noop certificate manager
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_create_authorized(self, mock_cert_data, **optionals):
+        cert1 = data_models.TLSContainer(certificate='cert 1')
+        cert2 = data_models.TLSContainer(certificate='cert 2')
+        cert3 = data_models.TLSContainer(certificate='cert 3')
+        mock_cert_data.return_value = {'tls_cert': cert1,
+                                       'sni_certs': [cert2, cert3]}
         sni1 = uuidutils.generate_uuid()
         sni2 = uuidutils.generate_uuid()
         lb_listener = {'name': 'listener1', 'default_pool_id': None,
@@ -690,21 +815,23 @@ class TestListener(base.BaseAPITest):
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
         self.assertEqual(self.NOT_AUTHORIZED_BODY, response.json)
 
-    def test_update_with_bad_handler(self):
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_update_with_bad_provider(self, mock_provider):
         api_listener = self.create_listener(
             constants.PROTOCOL_HTTP, 80,
             self.lb_id).get(self.root_tag)
         self.set_lb_status(lb_id=self.lb_id)
         new_listener = {'name': 'new_name'}
-        self.handler_mock().listener.update.side_effect = Exception()
-        self.put(self.LISTENER_PATH.format(listener_id=api_listener.get('id')),
-                 self._build_body(new_listener))
-        self.assert_correct_status(
-            lb_id=self.lb_id,
-            listener_id=api_listener.get('id'),
-            listener_prov_status=constants.ERROR)
+        mock_provider.side_effect = exceptions.ProviderDriverError(
+            prov='bad_driver', user_msg='broken')
+        response = self.put(
+            self.LISTENER_PATH.format(listener_id=api_listener.get('id')),
+            self._build_body(new_listener), status=500)
+        self.assertIn('Provider \'bad_driver\' reports error: broken',
+                      response.json.get('faultstring'))
 
-    def test_delete_with_bad_handler(self):
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_delete_with_bad_provider(self, mock_provider):
         api_listener = self.create_listener(
             constants.PROTOCOL_HTTP, 80,
             self.lb_id).get(self.root_tag)
@@ -714,20 +841,19 @@ class TestListener(base.BaseAPITest):
         api_listener['operating_status'] = constants.ONLINE
         response = self.get(self.LISTENER_PATH.format(
             listener_id=api_listener.get('id'))).json.get(self.root_tag)
-
         self.assertIsNone(api_listener.pop('updated_at'))
         self.assertIsNotNone(response.pop('updated_at'))
         self.assertEqual(api_listener, response)
-        self.handler_mock().listener.delete.side_effect = Exception()
+        mock_provider.side_effect = exceptions.ProviderDriverError(
+            prov='bad_driver', user_msg='broken')
         self.delete(self.LISTENER_PATH.format(
-            listener_id=api_listener.get('id')))
-        self.assert_correct_status(
-            lb_id=self.lb_id,
-            listener_id=api_listener.get('id'),
-            listener_prov_status=constants.ERROR)
+            listener_id=api_listener.get('id')), status=500)
 
-    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificate_data')
-    def test_update(self, mock_load_cert):
+    # TODO(johnsom) Fix this when there is a noop certificate manager
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_update(self, mock_cert_data):
+        cert1 = data_models.TLSContainer(certificate='cert 1')
+        mock_cert_data.return_value = {'tls_cert': cert1}
         tls_uuid = uuidutils.generate_uuid()
         listener = self.create_listener(
             constants.PROTOCOL_TCP, 80, self.lb_id,
@@ -737,15 +863,18 @@ class TestListener(base.BaseAPITest):
             default_pool_id=None).get(self.root_tag)
         self.set_lb_status(self.lb_id)
         new_listener = {'name': 'listener2', 'admin_state_up': True,
-                        'default_pool_id': self.pool_id}
+                        'default_pool_id': self.pool_id,
+                        'timeout_client_data': 1,
+                        'timeout_member_connect': 2,
+                        'timeout_member_data': 3,
+                        'timeout_tcp_inspect': 4}
         body = self._build_body(new_listener)
         listener_path = self.LISTENER_PATH.format(
             listener_id=listener['id'])
         api_listener = self.put(listener_path, body).json.get(self.root_tag)
-        update_expect = {'name': 'listener2', 'admin_state_up': True,
-                         'default_pool_id': self.pool_id,
-                         'provisioning_status': constants.PENDING_UPDATE,
+        update_expect = {'provisioning_status': constants.PENDING_UPDATE,
                          'operating_status': constants.ONLINE}
+        update_expect.update(new_listener)
         listener.update(update_expect)
         self.assertEqual(listener['created_at'], api_listener['created_at'])
         self.assertNotEqual(listener['updated_at'], api_listener['updated_at'])
@@ -755,6 +884,31 @@ class TestListener(base.BaseAPITest):
         self.assert_final_listener_statuses(self.lb_id,
                                             api_listener['id'])
 
+    def test_negative_update_udp_case(self):
+        api_listener = self.create_listener(constants.PROTOCOL_UDP, 6666,
+                                            self.lb_id).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        sni1 = uuidutils.generate_uuid()
+        sni2 = uuidutils.generate_uuid()
+        new_listener = {'name': 'new-listener',
+                        'admin_state_up': True,
+                        'connection_limit': 10,
+                        'default_tls_container_ref':
+                            uuidutils.generate_uuid(),
+                        'sni_container_refs': [sni1, sni2],
+                        'insert_headers': {
+                            "X-Forwarded-Port": "true",
+                            "X-Forwarded-For": "true"}}
+        listener_path = self.LISTENER_PATH.format(
+            listener_id=api_listener['id'])
+        expect_error_msg = (
+            "Validation failure: %s protocol listener does not support TLS or "
+            "header insertion.") % constants.PROTOCOL_UDP
+        res = self.put(listener_path, self._build_body(new_listener),
+                       status=400, expect_errors=True)
+        self.assertEqual(expect_error_msg, res.json['faultstring'])
+        self.assert_correct_status(lb_id=self.lb_id)
+
     def test_update_bad_listener_id(self):
         self.put(self.listener_path.format(listener_id='SEAN-CONNERY'),
                  body={}, status=404)
@@ -778,8 +932,11 @@ class TestListener(base.BaseAPITest):
         self.assert_final_listener_statuses(self.lb_id,
                                             listener['listener']['id'])
 
-    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificate_data')
-    def test_update_unset_defaults(self, mock_load_cert):
+    # TODO(johnsom) Fix this when there is a noop certificate manager
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_update_unset_defaults(self, mock_cert_data):
+        cert1 = data_models.TLSContainer(certificate='cert 1')
+        mock_cert_data.return_value = {'tls_cert': cert1}
         tls_uuid = uuidutils.generate_uuid()
         listener = self.create_listener(
             constants.PROTOCOL_TERMINATED_HTTPS, 80, self.lb_id,
@@ -787,22 +944,42 @@ class TestListener(base.BaseAPITest):
             admin_state_up=False, connection_limit=10,
             default_tls_container_ref=tls_uuid,
             default_pool_id=self.pool_id,
-            insert_headers={'X-Forwarded-For': 'true'}).get(self.root_tag)
+            insert_headers={'X-Forwarded-For': 'true'},
+            timeout_client_data=1, timeout_member_connect=2,
+            timeout_member_data=3, timeout_tcp_inspect=4).get(self.root_tag)
         self.set_lb_status(self.lb_id)
         unset_params = {
             'name': None, 'description': None, 'connection_limit': None,
             'default_tls_container_ref': None, 'sni_container_refs': None,
-            'insert_headers': None, 'default_pool_id': None}
+            'insert_headers': None, 'timeout_client_data': None,
+            'timeout_member_connect': None, 'timeout_member_data': None,
+            'timeout_tcp_inspect': None, 'default_pool_id': None}
         body = self._build_body(unset_params)
         listener_path = self.LISTENER_PATH.format(
             listener_id=listener['id'])
         api_listener = self.put(listener_path, body).json.get(self.root_tag)
 
+        self.assertEqual('', api_listener['name'])
+        self.assertEqual('', api_listener['description'])
+        self.assertEqual(constants.DEFAULT_CONNECTION_LIMIT,
+                         api_listener['connection_limit'])
+        self.assertIsNone(api_listener['default_tls_container_ref'])
         self.assertEqual([], api_listener['sni_container_refs'])
-        self.assertNotEqual(listener, api_listener)
+        self.assertEqual({}, api_listener['insert_headers'])
+        self.assertEqual(constants.DEFAULT_TIMEOUT_CLIENT_DATA,
+                         api_listener['timeout_client_data'])
+        self.assertEqual(constants.DEFAULT_TIMEOUT_MEMBER_CONNECT,
+                         api_listener['timeout_member_connect'])
+        self.assertEqual(constants.DEFAULT_TIMEOUT_MEMBER_DATA,
+                         api_listener['timeout_member_data'])
+        self.assertEqual(constants.DEFAULT_TIMEOUT_TCP_INSPECT,
+                         api_listener['timeout_tcp_inspect'])
+        self.assertIsNone(api_listener['default_pool_id'])
 
-    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificate_data')
-    def test_update_authorized(self, mock_load_cert):
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_update_authorized(self, mock_cert_data):
+        cert1 = data_models.TLSContainer(certificate='cert 1')
+        mock_cert_data.return_value = {'tls_cert': cert1}
         tls_uuid = uuidutils.generate_uuid()
         listener = self.create_listener(
             constants.PROTOCOL_TCP, 80, self.lb_id,
@@ -856,8 +1033,11 @@ class TestListener(base.BaseAPITest):
         self.assert_final_listener_statuses(self.lb_id,
                                             api_listener['id'])
 
-    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificate_data')
-    def test_update_not_authorized(self, mock_load_cert):
+    # TODO(johnsom) Fix this when there is a noop certificate manager
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_update_not_authorized(self, mock_cert_data):
+        cert1 = data_models.TLSContainer(certificate='cert 1')
+        mock_cert_data.return_value = {'tls_cert': cert1}
         tls_uuid = uuidutils.generate_uuid()
         listener = self.create_listener(
             constants.PROTOCOL_TCP, 80, self.lb_id,
@@ -1044,8 +1224,12 @@ class TestListener(base.BaseAPITest):
             listener_id=listener['listener'].get('id'))
         self.put(listener_path, {}, status=400)
 
-    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificate_data')
-    def test_update_bad_tls_ref(self, mock_load_cert):
+    # TODO(johnsom) Fix this when there is a noop certificate manager
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_update_bad_tls_ref(self, mock_cert_data):
+        cert2 = data_models.TLSContainer(certificate='cert 2')
+        cert3 = data_models.TLSContainer(certificate='cert 3')
+        mock_cert_data.return_value = {'sni_certs': [cert2, cert3]}
         sni1 = uuidutils.generate_uuid()
         sni2 = uuidutils.generate_uuid()
         tls_ref = uuidutils.generate_uuid()
@@ -1068,7 +1252,7 @@ class TestListener(base.BaseAPITest):
         body = self._build_body(lb_listener_put)
         listener_path = self.LISTENER_PATH.format(
             listener_id=api_listener['id'])
-        mock_load_cert.side_effect = [
+        self.cert_manager_mock().get_cert.side_effect = [
             Exception("bad cert"), None, Exception("bad cert")]
         response = self.put(listener_path, body, status=400).json
         self.assertIn(tls_ref2, response['faultstring'])
@@ -1122,6 +1306,28 @@ class TestListener(base.BaseAPITest):
             listener_id=api_listener['id'])
         self.put(listener_path, body, status=409)
 
+    def test_update_deleted(self):
+        lb = self.create_load_balancer(uuidutils.generate_uuid(),
+                                       name='lb1', description='desc1',
+                                       admin_state_up=False)
+        lb_id = lb['loadbalancer'].get('id')
+        self.set_lb_status(lb_id)
+        lb_listener = {'name': 'listener1', 'description': 'desc1',
+                       'admin_state_up': False,
+                       'protocol': constants.PROTOCOL_HTTP,
+                       'protocol_port': 80, 'connection_limit': 10,
+                       'loadbalancer_id': lb_id}
+        body = self._build_body(lb_listener)
+        api_listener = self.post(
+            self.LISTENERS_PATH, body).json.get(self.root_tag)
+        # This updates the child objects
+        self.set_lb_status(lb_id, status=constants.DELETED)
+        lb_listener_put = {'name': 'listener1_updated'}
+        body = self._build_body(lb_listener_put)
+        listener_path = self.LISTENER_PATH.format(
+            listener_id=api_listener['id'])
+        self.put(listener_path, body, status=404)
+
     def test_delete_pending_delete(self):
         lb = self.create_load_balancer(uuidutils.generate_uuid(),
                                        name='lb1', description='desc1',
@@ -1161,10 +1367,16 @@ class TestListener(base.BaseAPITest):
         self.set_lb_status(lb_id, status=constants.DELETED)
         listener_path = self.LISTENER_PATH.format(
             listener_id=api_listener['id'])
-        self.delete(listener_path, status=204)
+        self.delete(listener_path, status=404)
 
-    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificate_data')
-    def test_create_with_tls_termination_data(self, mock_load_cert):
+    # TODO(johnsom) Fix this when there is a noop certificate manager
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_create_with_tls_termination_data(self, mock_cert_data):
+        cert1 = data_models.TLSContainer(certificate='cert 1')
+        cert2 = data_models.TLSContainer(certificate='cert 2')
+        cert3 = data_models.TLSContainer(certificate='cert 3')
+        mock_cert_data.return_value = {'tls_cert': cert1,
+                                       'sni_certs': [cert2, cert3]}
         cert_id = uuidutils.generate_uuid()
         listener = self.create_listener(constants.PROTOCOL_TERMINATED_HTTPS,
                                         80, self.lb_id,
@@ -1174,8 +1386,11 @@ class TestListener(base.BaseAPITest):
         get_listener = self.get(listener_path).json['listener']
         self.assertEqual(cert_id, get_listener['default_tls_container_ref'])
 
-    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificate_data')
-    def test_update_with_tls_termination_data(self, mock_load_cert):
+    # TODO(johnsom) Fix this when there is a noop certificate manager
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_update_with_tls_termination_data(self, mock_cert_data):
+        cert1 = data_models.TLSContainer(certificate='cert 1')
+        mock_cert_data.return_value = {'tls_cert': cert1}
         cert_id = uuidutils.generate_uuid()
         listener = self.create_listener(constants.PROTOCOL_TERMINATED_HTTPS,
                                         80, self.lb_id)
@@ -1187,7 +1402,8 @@ class TestListener(base.BaseAPITest):
         self.put(listener_path,
                  self._build_body({'default_tls_container_ref': cert_id}))
         get_listener = self.get(listener_path).json['listener']
-        self.assertIsNone(get_listener.get('default_tls_container_ref'))
+        self.assertEqual(cert_id,
+                         get_listener.get('default_tls_container_ref'))
 
     def test_create_with_tls_termination_disabled(self):
         self.conf.config(group='api_settings',
@@ -1202,8 +1418,14 @@ class TestListener(base.BaseAPITest):
             .format(constants.PROTOCOL_TERMINATED_HTTPS),
             listener.get('faultstring'))
 
-    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificate_data')
-    def test_create_with_sni_data(self, mock_load_cert):
+    # TODO(johnsom) Fix this when there is a noop certificate manager
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_create_with_sni_data(self, mock_cert_data):
+        cert1 = data_models.TLSContainer(certificate='cert 1')
+        cert2 = data_models.TLSContainer(certificate='cert 2')
+        cert3 = data_models.TLSContainer(certificate='cert 3')
+        mock_cert_data.return_value = {'tls_cert': cert1,
+                                       'sni_certs': [cert2, cert3]}
         sni_id1 = uuidutils.generate_uuid()
         sni_id2 = uuidutils.generate_uuid()
         listener = self.create_listener(constants.PROTOCOL_HTTP, 80,
@@ -1215,8 +1437,12 @@ class TestListener(base.BaseAPITest):
         self.assertItemsEqual([sni_id1, sni_id2],
                               get_listener['sni_container_refs'])
 
-    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificate_data')
-    def test_update_with_sni_data(self, mock_load_cert):
+    # TODO(johnsom) Fix this when there is a noop certificate manager
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_update_with_sni_data(self, mock_cert_data):
+        cert2 = data_models.TLSContainer(certificate='cert 2')
+        cert3 = data_models.TLSContainer(certificate='cert 3')
+        mock_cert_data.return_value = {'sni_certs': [cert2, cert3]}
         sni_id1 = uuidutils.generate_uuid()
         sni_id2 = uuidutils.generate_uuid()
         listener = self.create_listener(constants.PROTOCOL_HTTP, 80,
@@ -1249,7 +1475,7 @@ class TestListener(base.BaseAPITest):
 
     def _getStats(self, listener_id):
         res = self.get(self.LISTENER_PATH.format(
-                       listener_id=listener_id + "/stats"))
+            listener_id=listener_id + "/stats"))
         return res.json.get('stats')
 
     def test_statistics(self):
@@ -1347,6 +1573,24 @@ class TestListener(base.BaseAPITest):
         with mock.patch.object(octavia.common.context.Context, 'project_id',
                                uuidutils.generate_uuid()):
             res = self.get(self.LISTENER_PATH.format(
-                           listener_id=li['id'] + "/stats"), status=403)
+                listener_id=li['id'] + "/stats"), status=403)
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
         self.assertEqual(self.NOT_AUTHORIZED_BODY, res.json)
+
+    def test_statistics_get_deleted(self):
+        lb = self.create_load_balancer(
+            uuidutils.generate_uuid()).get('loadbalancer')
+        self.set_lb_status(lb['id'])
+        li = self.create_listener(
+            constants.PROTOCOL_HTTP, 80, lb.get('id')).get('listener')
+        amphora = self.create_amphora(uuidutils.generate_uuid(), lb['id'])
+        self.create_listener_stats_dynamic(
+            listener_id=li.get('id'),
+            amphora_id=amphora.id,
+            bytes_in=random.randint(1, 9),
+            bytes_out=random.randint(1, 9),
+            total_connections=random.randint(1, 9),
+            request_errors=random.randint(1, 9))
+        self.set_lb_status(lb['id'], status=constants.DELETED)
+        self.get(self.LISTENER_PATH.format(
+            listener_id=li.get('id') + "/stats"), status=404)
