@@ -15,10 +15,13 @@
 
 import collections
 
+from oslo_config import cfg
 
 from octavia.common import constants
 from octavia.tests.unit.common.sample_configs import sample_certs
 
+CONF = cfg.CONF
+
 
 def sample_amphora_tuple():
     in_amphora = collections.namedtuple(
@@ -74,7 +77,8 @@ RET_MEMBER_1 = {
     'enabled': True,
     'operating_status': 'ACTIVE',
     'monitor_address': None,
-    'monitor_port': None}
+    'monitor_port': None,
+    'backup': False}
 
 RET_MEMBER_2 = {
     'id': 'sample_member_id_2',
@@ -85,7 +89,8 @@ RET_MEMBER_2 = {
     'enabled': True,
     'operating_status': 'ACTIVE',
     'monitor_address': None,
-    'monitor_port': None}
+    'monitor_port': None,
+    'backup': False}
 
 RET_MEMBER_3 = {
     'id': 'sample_member_id_3',
@@ -96,7 +101,8 @@ RET_MEMBER_3 = {
     'enabled': True,
     'operating_status': 'ACTIVE',
     'monitor_address': None,
-    'monitor_port': None}
+    'monitor_port': None,
+    'backup': False}
 
 RET_POOL_1 = {
     'id': 'sample_pool_id_1',
@@ -242,7 +248,12 @@ RET_LISTENER = {
     'pools': [RET_POOL_1],
     'l7policies': [],
     'enabled': True,
-    'insert_headers': {}}
+    'insert_headers': {},
+    'timeout_client_data': 50000,
+    'timeout_member_connect': 5000,
+    'timeout_member_data': 50000,
+    'timeout_tcp_inspect': 0,
+}
 
 RET_LISTENER_L7 = {
     'id': 'sample_listener_id_1',
@@ -258,7 +269,12 @@ RET_LISTENER_L7 = {
     'l7policies': [RET_L7POLICY_1, RET_L7POLICY_2, RET_L7POLICY_3,
                    RET_L7POLICY_4, RET_L7POLICY_5, RET_L7POLICY_6],
     'enabled': True,
-    'insert_headers': {}}
+    'insert_headers': {},
+    'timeout_client_data': 50000,
+    'timeout_member_connect': 5000,
+    'timeout_member_data': 50000,
+    'timeout_tcp_inspect': 0,
+}
 
 RET_LISTENER_TLS = {
     'id': 'sample_listener_id_1',
@@ -323,6 +339,100 @@ RET_LB_L7 = {
     'enabled': True,
     'global_connection_limit': constants.HAPROXY_MAX_MAXCONN}
 
+UDP_SOURCE_IP_BODY = {
+    'type': constants.SESSION_PERSISTENCE_SOURCE_IP,
+    'persistence_timeout': 33,
+    'persistence_granularity': '255.0.0.0'
+}
+
+RET_UDP_HEALTH_MONITOR = {
+    'id': 'sample_monitor_id_1',
+    'type': constants.HEALTH_MONITOR_UDP_CONNECT,
+    'delay': 30,
+    'timeout': 31,
+    'enabled': True,
+    'fall_threshold': 3,
+    'check_script_path': (CONF.haproxy_amphora.base_path +
+                          '/lvs/check/udp_check.sh')
+}
+
+UDP_HEALTH_MONITOR_NO_SCRIPT = {
+    'id': 'sample_monitor_id_1',
+    'check_script_path': None,
+    'delay': 30,
+    'enabled': True,
+    'fall_threshold': 3,
+    'timeout': 31,
+    'type': 'UDP'
+}
+
+RET_UDP_MEMBER = {
+    'id': 'member_id_1',
+    'address': '192.0.2.10',
+    'protocol_port': 82,
+    'weight': 13,
+    'enabled': True,
+    'monitor_address': None,
+    'monitor_port': None
+}
+
+RET_UDP_MEMBER_MONITOR_IP_PORT = {
+    'id': 'member_id_1',
+    'address': '192.0.2.10',
+    'protocol_port': 82,
+    'weight': 13,
+    'enabled': True,
+    'monitor_address': '192.168.1.1',
+    'monitor_port': 9000
+}
+
+UDP_MEMBER_1 = {
+    'id': 'sample_member_id_1',
+    'address': '10.0.0.99',
+    'enabled': True,
+    'protocol_port': 82,
+    'weight': 13,
+    'monitor_address': None,
+    'monitor_port': None
+}
+
+UDP_MEMBER_2 = {
+    'id': 'sample_member_id_2',
+    'address': '10.0.0.98',
+    'enabled': True,
+    'protocol_port': 82,
+    'weight': 13,
+    'monitor_address': None,
+    'monitor_port': None
+}
+
+RET_UDP_POOL = {
+    'id': 'sample_pool_id_1',
+    'enabled': True,
+    'health_monitor': UDP_HEALTH_MONITOR_NO_SCRIPT,
+    'lb_algorithm': 'rr',
+    'members': [UDP_MEMBER_1, UDP_MEMBER_2],
+    'protocol': 'udp',
+    'session_persistence': UDP_SOURCE_IP_BODY
+}
+
+RET_UDP_LISTENER = {
+    'connection_limit': 98,
+    'default_pool': {
+        'id': 'sample_pool_id_1',
+        'enabled': True,
+        'health_monitor': RET_UDP_HEALTH_MONITOR,
+        'lb_algorithm': 'rr',
+        'members': [UDP_MEMBER_1, UDP_MEMBER_2],
+        'protocol': 'udp',
+        'session_persistence': UDP_SOURCE_IP_BODY
+    },
+    'enabled': True,
+    'id': 'sample_listener_id_1',
+    'protocol_mode': 'udp',
+    'protocol_port': '80'
+}
+
 
 def sample_loadbalancer_tuple(proto=None, monitor=True, persistence=True,
                               persistence_type=None, tls=False, sni=False,
@@ -387,13 +497,19 @@ def sample_vip_tuple():
     return vip(ip_address='10.0.0.2')
 
 
-def sample_listener_tuple(proto=None, monitor=True, persistence=True,
-                          persistence_type=None, persistence_cookie=None,
+def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
+                          persistence=True, persistence_type=None,
+                          persistence_cookie=None, persistence_timeout=None,
+                          persistence_granularity=None,
                           tls=False, sni=False, peer_port=None, topology=None,
                           l7=False, enabled=True, insert_headers=None,
                           be_proto=None, monitor_ip_port=False,
-                          monitor_proto=None, connection_limit=-1,
-                          disabled_member=False):
+                          monitor_proto=None, backup_member=False,
+                          disabled_member=False, connection_limit=-1,
+                          timeout_client_data=50000,
+                          timeout_member_connect=5000,
+                          timeout_member_data=50000,
+                          timeout_tcp_inspect=0):
     proto = 'HTTP' if proto is None else proto
     if be_proto is None:
         be_proto = 'HTTP' if proto is 'TERMINATED_HTTPS' else proto
@@ -406,7 +522,9 @@ def sample_listener_tuple(proto=None, monitor=True, persistence=True,
                     'connection_limit, tls_certificate_id, '
                     'sni_container_ids, default_tls_container, '
                     'sni_containers, load_balancer, peer_port, pools, '
-                    'l7policies, enabled, insert_headers',)
+                    'l7policies, enabled, insert_headers, timeout_client_data,'
+                    'timeout_member_connect, timeout_member_data, '
+                    'timeout_tcp_inspect',)
     if l7:
         pools = [
             sample_pool_tuple(
@@ -433,7 +551,7 @@ def sample_listener_tuple(proto=None, monitor=True, persistence=True,
                 persistence_type=persistence_type,
                 persistence_cookie=persistence_cookie,
                 monitor_ip_port=monitor_ip_port, monitor_proto=monitor_proto,
-                disabled_member=disabled_member)]
+                backup_member=backup_member, disabled_member=disabled_member)]
         l7policies = []
     return in_listener(
         id='sample_listener_id_1',
@@ -447,7 +565,10 @@ def sample_listener_tuple(proto=None, monitor=True, persistence=True,
             proto=be_proto, monitor=monitor, persistence=persistence,
             persistence_type=persistence_type,
             persistence_cookie=persistence_cookie,
-            monitor_ip_port=monitor_ip_port, monitor_proto=monitor_proto),
+            persistence_timeout=persistence_timeout,
+            persistence_granularity=persistence_granularity,
+            monitor_ip_port=monitor_ip_port,
+            monitor_proto=monitor_proto) if alloc_default_pool else '',
         connection_limit=connection_limit,
         tls_certificate_id='cont_id_1' if tls else '',
         sni_container_ids=['cont_id_2', 'cont_id_3'] if sni else [],
@@ -476,7 +597,11 @@ def sample_listener_tuple(proto=None, monitor=True, persistence=True,
         pools=pools,
         l7policies=l7policies,
         enabled=enabled,
-        insert_headers=insert_headers
+        insert_headers=insert_headers,
+        timeout_client_data=timeout_client_data,
+        timeout_member_connect=timeout_member_connect,
+        timeout_member_data=timeout_member_data,
+        timeout_tcp_inspect=timeout_tcp_inspect
     )
 
 
@@ -504,16 +629,24 @@ def sample_tls_container_tuple(id='cont_id_1', certificate=None,
 
 def sample_pool_tuple(proto=None, monitor=True, persistence=True,
                       persistence_type=None, persistence_cookie=None,
+                      persistence_timeout=None, persistence_granularity=None,
                       sample_pool=1, monitor_ip_port=False,
-                      monitor_proto=None, disabled_member=False):
+                      monitor_proto=None, backup_member=False,
+                      disabled_member=False):
     proto = 'HTTP' if proto is None else proto
     monitor_proto = proto if monitor_proto is None else monitor_proto
     in_pool = collections.namedtuple(
         'pool', 'id, protocol, lb_algorithm, members, health_monitor,'
                 'session_persistence, enabled, operating_status')
-    persis = sample_session_persistence_tuple(
-        persistence_type=persistence_type,
-        persistence_cookie=persistence_cookie) if persistence is True else None
+    if (proto == constants.PROTOCOL_UDP and
+            persistence_type == constants.SESSION_PERSISTENCE_SOURCE_IP):
+        kwargs = {'persistence_type': persistence_type,
+                  'persistence_timeout': persistence_timeout,
+                  'persistence_granularity': persistence_granularity}
+    else:
+        kwargs = {'persistence_type': persistence_type,
+                  'persistence_cookie': persistence_cookie}
+    persis = sample_session_persistence_tuple(**kwargs)
     mon = None
     if sample_pool == 1:
         id = 'sample_pool_id_1'
@@ -521,6 +654,7 @@ def sample_pool_tuple(proto=None, monitor=True, persistence=True,
                                        monitor_ip_port=monitor_ip_port),
                    sample_member_tuple('sample_member_id_2', '10.0.0.98',
                                        monitor_ip_port=monitor_ip_port,
+                                       backup=backup_member,
                                        enabled=not disabled_member)]
         if monitor is True:
             mon = sample_health_monitor_tuple(proto=monitor_proto)
@@ -536,18 +670,19 @@ def sample_pool_tuple(proto=None, monitor=True, persistence=True,
         lb_algorithm='ROUND_ROBIN',
         members=members,
         health_monitor=mon,
-        session_persistence=persis,
+        session_persistence=persis if persistence is True else None,
         enabled=True,
         operating_status='ACTIVE')
 
 
 def sample_member_tuple(id, ip, enabled=True, operating_status='ACTIVE',
-                        monitor_ip_port=False):
+                        monitor_ip_port=False, backup=False):
     in_member = collections.namedtuple('member',
                                        'id, ip_address, protocol_port, '
                                        'weight, subnet_id, '
                                        'enabled, operating_status, '
-                                       'monitor_address, monitor_port')
+                                       'monitor_address, monitor_port, '
+                                       'backup')
     monitor_address = '192.168.1.1' if monitor_ip_port else None
     monitor_port = 9000 if monitor_ip_port else None
     return in_member(
@@ -559,23 +694,31 @@ def sample_member_tuple(id, ip, enabled=True, operating_status='ACTIVE',
         enabled=enabled,
         operating_status=operating_status,
         monitor_address=monitor_address,
-        monitor_port=monitor_port)
+        monitor_port=monitor_port,
+        backup=backup)
 
 
 def sample_session_persistence_tuple(persistence_type=None,
-                                     persistence_cookie=None):
+                                     persistence_cookie=None,
+                                     persistence_timeout=None,
+                                     persistence_granularity=None):
     spersistence = collections.namedtuple('SessionPersistence',
-                                          'type, cookie_name')
+                                          'type, cookie_name, '
+                                          'persistence_timeout, '
+                                          'persistence_granularity')
     pt = 'HTTP_COOKIE' if persistence_type is None else persistence_type
     return spersistence(type=pt,
-                        cookie_name=persistence_cookie)
+                        cookie_name=persistence_cookie,
+                        persistence_timeout=persistence_timeout,
+                        persistence_granularity=persistence_granularity)
 
 
 def sample_health_monitor_tuple(proto='HTTP', sample_hm=1):
     proto = 'HTTP' if proto is 'TERMINATED_HTTPS' else proto
     monitor = collections.namedtuple(
         'monitor', 'id, type, delay, timeout, fall_threshold, rise_threshold,'
-                   'http_method, url_path, expected_codes, enabled')
+                   'http_method, url_path, expected_codes, enabled, '
+                   'check_script_path')
 
     if sample_hm == 1:
         id = 'sample_monitor_id_1'
@@ -583,10 +726,24 @@ def sample_health_monitor_tuple(proto='HTTP', sample_hm=1):
     elif sample_hm == 2:
         id = 'sample_monitor_id_2'
         url_path = '/healthmon.html'
-    return monitor(id=id, type=proto, delay=30,
-                   timeout=31, fall_threshold=3, rise_threshold=2,
-                   http_method='GET', url_path=url_path,
-                   expected_codes='418', enabled=True)
+    kwargs = {
+        'id': id,
+        'type': proto,
+        'delay': 30,
+        'timeout': 31,
+        'fall_threshold': 3,
+        'rise_threshold': 2,
+        'http_method': 'GET',
+        'url_path': url_path,
+        'expected_codes': '418',
+        'enabled': True
+    }
+    if proto == constants.HEALTH_MONITOR_UDP_CONNECT:
+        kwargs['check_script_path'] = (CONF.haproxy_amphora.base_path +
+                                       'lvs/check/' + 'udp_check.sh')
+    else:
+        kwargs['check_script_path'] = None
+    return monitor(**kwargs)
 
 
 def sample_l7policy_tuple(id,
@@ -685,48 +842,50 @@ def sample_l7rule_tuple(id,
 
 
 def sample_base_expected_config(frontend=None, backend=None,
-                                peers=None, global_opts=None):
+                                peers=None, global_opts=None, defaults=None):
     if frontend is None:
         frontend = ("frontend sample_listener_id_1\n"
                     "    option httplog\n"
                     "    maxconn {maxconn}\n"
                     "    bind 10.0.0.2:80\n"
                     "    mode http\n"
-                    "    default_backend sample_pool_id_1\n\n").format(
+                    "    default_backend sample_pool_id_1\n"
+                    "    timeout client 50000\n\n").format(
             maxconn=constants.HAPROXY_MAX_MAXCONN)
     if backend is None:
         backend = ("backend sample_pool_id_1\n"
                    "    mode http\n"
                    "    balance roundrobin\n"
                    "    cookie SRV insert indirect nocache\n"
-                   "    timeout check 31\n"
+                   "    timeout check 31s\n"
                    "    option httpchk GET /index.html\n"
                    "    http-check expect rstatus 418\n"
                    "    fullconn {maxconn}\n"
+                   "    option allbackups\n"
+                   "    timeout connect 5000\n"
+                   "    timeout server 50000\n"
                    "    server sample_member_id_1 10.0.0.99:82 weight 13 "
                    "check inter 30s fall 3 rise 2 cookie sample_member_id_1\n"
                    "    server sample_member_id_2 10.0.0.98:82 weight 13 "
                    "check inter 30s fall 3 rise 2 cookie sample_member_id_2\n"
                    "\n").format(maxconn=constants.HAPROXY_MAX_MAXCONN)
+
     if peers is None:
         peers = "\n\n"
     if global_opts is None:
         global_opts = "    maxconn {maxconn}\n\n".format(
             maxconn=constants.HAPROXY_MAX_MAXCONN)
+    if defaults is None:
+        defaults = ("defaults\n"
+                    "    log global\n"
+                    "    retries 3\n"
+                    "    option redispatch\n\n")
     return ("# Configuration for loadbalancer sample_loadbalancer_id_1\n"
             "global\n"
             "    daemon\n"
             "    user nobody\n"
-            "    group nogroup\n"
             "    log /dev/log local0\n"
             "    log /dev/log local1 notice\n"
             "    stats socket /var/lib/octavia/sample_listener_id_1.sock"
             " mode 0666 level user\n" +
-            global_opts +
-            "defaults\n"
-            "    log global\n"
-            "    retries 3\n"
-            "    option redispatch\n"
-            "    timeout connect 5000\n"
-            "    timeout client 50000\n"
-            "    timeout server 50000\n\n" + peers + frontend + backend)
+            global_opts + defaults + peers + frontend + backend)
